/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer = function (aContext) {
    "use strict";

/*
 * Initialization:
 * 
 * 1) Check if MediaSource is available.
 * 2) Load manifest.
 * 3) Parse manifest.
 * 4) Check if Video Element can play codecs.
 * 5) Register MediaSource with Video Element.
 * 6) Create SourceBuffers.
 * 7) Do live stuff.
 *      a. Start manifest refresh.
 *      b. Calculate live point.
 *      c. Calculate offset between availabilityStartTime and initial video timestamp.
 * 8) Start buffer managers.
 * 
 * Buffer Management:
 * 
 * 1) Generate metrics.
 * 2) Check if fragments should be loaded.
 * 3) Check ABR for change in quality.
 * 4) Figure out which fragments to load.
 * 5) Load fragments.
 * 6) Transform fragments.
 * 7) Push fragmemt bytes into SourceBuffer.
 */
    
    var context = aContext,
        system,
        element,
        source,
        model,
        stream,
        initialized = false,
        playing = false,
        autoPlay = true,
        
        isReady = function () {
            return (element !== undefined && source !== undefined);
        },
        
        play = function () {
            if (!initialized) {
                throw "MediaPlayer not initialized!";
            }
            
            if (!this.capabilities.supportsMediaSource()) {
                throw "Media Source not supported.";
            }
            
            if (!element || !source) {
                throw "Missing view or source.";
            }
            
            playing = true;
            this.debug.log("Playback initiated!");
            stream = system.getObject("stream");
            stream.load(source);
        },
        
        doAutoPlay = function () {
            if (autoPlay && isReady()) {
                play.call(this);
            }
        };

    // Set up DI.
    system = new dijon.System();
    system.mapValue("system", system);
    system.mapOutlet("system");
    system.injectInto(context);
    
    return {
        debug: undefined,
        capabilities: undefined,
        videoModel: undefined,
        
        startup: function () {
            if (!initialized) {
                system.injectInto(this);
                initialized = true;
            }
        },
        
        setAutoPlay: function(value) {
            autoPlay = value;
        },
        
        getAutoPlay: function() {
            return autoPlay;
        },
        
        attachView: function (view) {
            if (!initialized) {
                throw "MediaPlayer not initialized!";
            }
            
            element = view;
            model = new MediaPlayer.models.VideoModel(element);
            this.videoModel.setElement(element);
            
            // TODO : update
            
            if (!playing) {
                doAutoPlay.call(this);
            }
        },
        
        attachSource: function (url) {
            if (!initialized) {
                throw "MediaPlayer not initialized!";
            }
            
            source = url;
            
            // TODO : update
            
            if (!playing) {
                doAutoPlay.call(this);
            }
        },
        
        play: play
    };
};

MediaPlayer.prototype = {
    constructor: MediaPlayer
};

MediaPlayer.dependencies = {};
MediaPlayer.utils = {};
MediaPlayer.models = {};
MediaPlayer.vo = {};
MediaPlayer.vo.metrics = {};
MediaPlayer.rules = {};
MediaPlayer.di = {};

/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.di.Context = function () {
    return {
        system : undefined,
        setup : function() {
            this.system.autoMapOutlets = true;
            
            this.system.mapSingleton('debug', MediaPlayer.utils.Debug);
            this.system.mapSingleton('capabilities', MediaPlayer.utils.Capabilities);
            this.system.mapSingleton('videoModel', MediaPlayer.models.VideoModel);
            
            this.system.mapSingleton('mediaSourceExt', MediaPlayer.dependencies.MediaSourceExtensions);
            this.system.mapSingleton('sourceBufferExt', MediaPlayer.dependencies.SourceBufferExtensions);
            this.system.mapSingleton('bufferExt', MediaPlayer.dependencies.BufferExtensions);
            
            this.system.mapSingleton('abrController', MediaPlayer.dependencies.AbrController);
            this.system.mapClass('bandwidthRule', MediaPlayer.rules.BandwidthRule);
            this.system.mapClass('abrRulesCollection', MediaPlayer.rules.BaseRulesCollection);
            
            this.system.mapClass('bufferController', MediaPlayer.dependencies.BufferController);
            this.system.mapClass('manifestLoader', MediaPlayer.dependencies.ManifestLoader);
            this.system.mapClass('fragmentController', MediaPlayer.dependencies.FragmentController);
            this.system.mapClass('fragmentLoader', MediaPlayer.dependencies.FragmentLoader);
            this.system.mapClass('stream', MediaPlayer.dependencies.Stream);
        }
    };
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.utils.Capabilities = function () {
    "use strict";
};

MediaPlayer.utils.Capabilities.prototype = {
    constructor: MediaPlayer.utils.Capabilities,
    
    supportsMediaSource: function() {
        var hasWebKit = (window.WebKitMediaSource !== null),
            hasMediaSource = (window.MediaSource !== null);
        
        return (hasWebKit || hasMediaSource);
    },
    
    supportsCodec: function (element, codec) {
        if (!(element instanceof HTMLVideoElement)) {
            throw "element must be of type HTMLVideoElement.";
        }
        
        var canPlay = element.canPlayType(codec);
        return (canPlay === "probably");
    }
};
﻿/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.utils.Debug = function () {
    "use strict";
    
    var htmlConsole = null;
    
    return {
        init: function (hc) {
            htmlConsole = hc;
        },
        
        log: function (message) {
            console.log(message);
            
            if (htmlConsole !== null) {
                var output = message + "</br>" + htmlConsole.innerHTML;
                htmlConsole.innerHTML = output;
            }
        }
    };
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.models.VideoModel = function () {
    "use strict";
    
    var element;
    
    return {
        play: function () {
            element.play();
        },
        
        pause: function () {
            element.pause();
        },
        
        getCurrentTime: function () {
            return element.currentTime;
        },
        
        listen: function (type, callback) {
            element.addEventListener(type, callback, false);
        },
        
        getElement: function () {
            return element;
        },
        
        setElement: function (value) {
            element = value;
        },
        
        setSource: function (source) {
            element.src = source;
        }
    };
};

MediaPlayer.models.VideoModel.prototype = {
    constructor: MediaPlayer.models.VideoModel
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
MediaPlayer.vo.SegmentRequest = function () {
    "use strict";
    this.action = "download";
    this.startTime = NaN;
    this.duration = NaN;
    this.range = null;
    this.url = null;
    this.requestStartDate = null;
    this.requestEndDate = null;
    this.deferred = null;
};

MediaPlayer.vo.SegmentRequest.prototype = {
    constructor: MediaPlayer.vo.SegmentRequest,
    ACTION_DOWNLOAD: "download",
    ACTION_COMPLETE: "complete"
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.ManifestLoader = function () {
    "use strict";

    var parseBaseUrl = function (url) {
            var base = null;
            
            if (url.indexOf("/") !== -1) {
                base = url.substring(0, url.lastIndexOf("/") + 1);
            }
            
            return base;
        },
        
        load = function (url) {
            var baseUrl = parseBaseUrl(url),
                deferred = Q.defer(),
                request = new XMLHttpRequest(),
                self = this;
            
            this.debug.log("Start loading manifest: " + url);
            
            request.open("GET", url, true);
            
            request.onload = function () {
                self.parser.parse(request.responseText, baseUrl).then(
                    function (manifest) {
                        deferred.resolve(manifest);
                    }
                );
            };
            
            request.onerror = function () {
                deferred.reject("Error loading manifest.");
            };
            
            request.send();  
            
            return deferred.promise;
        };
    
    return {
        debug: undefined,
        parser: undefined,
        load: load
    };
};

MediaPlayer.dependencies.ManifestLoader.prototype = {
    constructor: MediaPlayer.dependencies.ManifestLoader
};



/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.MediaSourceExtensions = function () {
    "use strict";
};

MediaPlayer.dependencies.MediaSourceExtensions.prototype = {
    constructor: MediaPlayer.dependencies.MediaSourceExtensions,
    
    createMediaSource: function () {
        var hasWebKit = (window.WebKitMediaSource !== null),
            hasMediaSource = (window.MediaSource !== null);
        
        if (hasWebKit) {
            return Q.when(new WebKitMediaSource());
        }
        if (hasMediaSource) {
            return Q.when(new MediaSource());
        }
        
        return null;
    },
    
    attachMediaSource: function (source, videoModel) {
        videoModel.setSource(window.URL.createObjectURL(source));
        return Q.when(true);
    },
    
    setDuration: function (source, value) {
        source.duration = value;
        return Q.when(source.duration);
    }
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.SourceBufferExtensions = function () {
    "use strict";
};

MediaPlayer.dependencies.SourceBufferExtensions.prototype = {
    constructor: MediaPlayer.dependencies.SourceBufferExtensions,
    
    createSourceBuffer: function (mediaSource, codec) {
        return Q.when(mediaSource.addSourceBuffer(codec));
    },
    
    getBufferLength: function (buffer, time) {
        var ranges = null,
            rangeIndex = -1,
            len,
            i;
            
        ranges = buffer.buffered;

        if (ranges !== null) {
            for (i = 0, len = ranges.length; i < len; i += 1) {
                if (time >= ranges.start(i) && time < ranges.end(i)) {
                    rangeIndex = i;
                }
            }
        }
        
        if (rangeIndex === -1) {
            return Q.when(0);
        } else {
            return Q.when(ranges.end(rangeIndex) - time);
        }
    },
    
    append: function (buffer, bytes) {
        buffer.append(bytes);
        return Q.when(true);
    }
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.BufferExtensions = function () {
    "use strict";
    
    var bufferTime;
    
    return {
        decideBufferLength: function(minBufferTime) {
            if (isNaN(minBufferTime) || minBufferTime <= 0) {
                bufferTime = 4;
            } else {
                bufferTime = minBufferTime;
            }
            return Q.when(bufferTime);
        },
        
        shouldBufferMore: function (bufferLength, metrics) {
            var result = (bufferLength < bufferTime);
            return Q.when(result);
        }
    };
};

MediaPlayer.dependencies.BufferExtensions.prototype = {
    constructor: MediaPlayer.dependencies.BufferExtensions
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.FragmentController = function () {
    "use strict";
};

MediaPlayer.dependencies.FragmentController.prototype = {
    constructor: MediaPlayer.dependencies.FragmentController,
    
    process: function (bytes) {
        var result = null;
        
        if (bytes !== null && bytes.byteLength > 0) {
            result = new Uint8Array(bytes);
        }
        
        return Q.when(result);
    }
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.AbrController = function () {
    "use strict";
    
    var autoSwitchBitrate = false,
        quality = 0;
    
    return {
        debug: undefined,
        abrRulesCollection: undefined,
        
        getAutoSwitchBitrate: function () {
            return autoSwitchBitrate;
        },
        
        getPlaybackQuality: function (metrics, data) {
            var newQuality = 999,
                ruleQuality,
                rules,
                i,
                len;
            
            this.debug.log("ABR enabled? (" + autoSwitchBitrate + ")");
            if (autoSwitchBitrate) {
                this.debug.log("Check ABR rules.");
                rules = this.abrRulesCollection.getRules();
                
                for (i = 0, len = rules.length; i < len; i += 1) {
                    ruleQuality = rules[i].checkIndex(metrics);
                    newQuality = Math.min(newQuality, ruleQuality);
                }
                
                quality = newQuality;
            }
            
            this.debug.log("Returning quality of " + newQuality);
            return Q.when(quality);
        }
    };
};

MediaPlayer.dependencies.AbrController.prototype = {
    constructor: MediaPlayer.dependencies.AbrController
};
﻿/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.FragmentLoader = function () {
    "use strict";

    var requests = [],
        lastRequest = null,
        loading = false,
        
        loadNext = function () {
            var req = new XMLHttpRequest();

            if (requests.length > 0) {
                lastRequest = requests.shift();
                lastRequest.requestStartDate = new Date();
                loading = true;
                
                req.responseType = "arraybuffer";
                req.open("GET", lastRequest.url, true);
                if (lastRequest.range) {
                    req.setRequestHeader("Range", "bytes=" + lastRequest.range);
                }
                
                req.onload = function () {
                    var bytes = req.response;
                    lastRequest.requestEndDate = new Date();
                    lastRequest.deferred.resolve({
                        data: bytes,
                        request: lastRequest
                    });
                    loadNext();
                };
                
                req.onerror = function () {
                    lastRequest.deferred.reject("Error loading fragment.");
                };
                
                req.send();
            } else {
                loading = false;
            }
        },
        
        loadRequest = function (req) {
            var deferred = Q.defer();
            req.deferred = deferred;
            
            requests.push(req);
            
            if (!loading) {
                loadNext();
            }
            
            return deferred.promise;
        };

    return {
        getLoading: function () {
            return loading;
        },
        
        load: function (req) {
            if (!req) {
                return;
            }
            
            return loadRequest(req);
        }
    };
};

MediaPlayer.dependencies.FragmentLoader.prototype = {
    constructor: MediaPlayer.dependencies.FragmentLoader
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.Stream = function () {
    "use strict";

    var manifest,
        mediaSource,
        videoController = null,
        audioController = null,
        autoPlay = true,
        initialized = false,
        
        play = function () {
            this.debug.log("Attempting play...");
            
            if (!initialized) {
                return;
            }
            
            this.debug.log("Do play.");
            this.videoModel.play();
        },
        
        pause = function () {
            this.debug.log("Do pause.");
            this.videoModel.pause();
        },
        
        seek = function (time) {
            this.debug.log("Attempting seek...");
            
            if (!initialized) {
                return;
            }
            
            this.debug.log("Do seek: " + time);
            
            if (videoController) {
                videoController.start();
            }
            if (audioController) {
                audioController.start();
            }
        },
        
        setUpMediaSource = function () {
            // Error handling
            var deferred = Q.defer(),
                self = this,
            
                onMediaSourceClose = function (e) {
                    pause.call(self);
                    self.debug.log("Error initializing MediaSource.");
                    self.debug.log(e);
                    alert("Error: Media Source Close.");
                },
            
                onMediaSourceOpen = function (e) {
                    self.debug.log("MediaSource is open!");
                    
                    mediaSource.removeEventListener("sourceopen", onMediaSourceOpen);
                    mediaSource.removeEventListener("webkitsourceopen", onMediaSourceOpen);
                    
                    deferred.resolve(mediaSource);
                };
            
            this.debug.log("MediaSource should be closed. (" + mediaSource.readyState + ")");
            
            mediaSource.addEventListener("sourceclose", onMediaSourceClose, false);
            mediaSource.addEventListener("webkitsourceclose", onMediaSourceClose, false);
            
            mediaSource.addEventListener("sourceopen", onMediaSourceOpen, false);
            mediaSource.addEventListener("webkitsourceopen", onMediaSourceOpen, false);
            
            this.mediaSourceExt.attachMediaSource(mediaSource, this.videoModel);
            this.debug.log("MediaSource attached to video.  Waiting on open...");
            
            return deferred.promise;
        },
        
        checkIfInitialized = function (videoReady, audioReady, deferred) {
            if (videoReady && audioReady) {
                if (videoController === null && audioController === null) {
                    var msg = "No streams to play.";
                    alert(msg);
                    this.debug.log(msg);
                    deferred.reject(msg);
                } else {
                    this.debug.log("MediaSource initialized!");
                    deferred.resolve(true);
                }
            }
        },
        
        initializeMediaSource = function () {
            this.debug.log("Getting MediaSource ready...");
            
            var initialize = Q.defer(),
                videoReady = false,
                audioReady = false,
                minBufferTime,
                self = this;
            
            // Figure out some bits about the stream before building anything.
            self.manifestExt.getIsLive(manifest).then(
                function (isLive) {
                    self.debug.log("Gathering information for buffers. (1)");
                    self.manifestExt.getDuration(manifest).then(
                        function (duration) {
                            self.debug.log("Gathering information for buffers. (2)");
                            self.bufferExt.decideBufferLength(manifest.minBufferTime).then(
                                function (time) {
                                    self.debug.log("Gathering information for buffers. (3)");
                                    self.debug.log("Buffer time: " + time);
                                    minBufferTime = time;
                                    return self.manifestExt.getVideoData(manifest);
                                }
                            ).then(
                                function (videoData) {
                                    if (videoData !== null) {
                                        self.debug.log("Create video buffer.");
                                        self.manifestExt.getCodec(videoData).then(
                                            function (codec) {
                                                self.debug.log("Video codec: " + codec);
                                                if (!self.capabilities.supportsCodec(self.videoModel.getElement(), codec)) {
                                                    self.debug.log("Codec (" + codec + ") is not supported.");
                                                    alert("Codec (" + codec + ") is not supported.");
                                                    return Q.when(null);
                                                } else {
                                                    return self.sourceBufferExt.createSourceBuffer(mediaSource, codec);
                                                }
                                            }
                                        ).then(
                                            function (buffer) {
                                                if (buffer === null) {
                                                    self.debug.log("No buffer was created, skipping video stream.");
                                                } else {
                                                    // TODO : How to tell index handler live/duration?
                                                    // TODO : Pass to controller and then pass to each method on handler?
                                                    
                                                    videoController = self.system.getObject("bufferController");
                                                    videoController.setType("video");
                                                    videoController.setData(videoData);
                                                    videoController.setBuffer(buffer);
                                                    videoController.setMinBufferTime(minBufferTime);
                                                    
                                                    self.debug.log("Video is ready!");
                                                }
                                                
                                                videoReady = true;
                                                checkIfInitialized.call(self, videoReady, audioReady, initialize);
                                            },
                                            function (error) {
                                                alert("Error creating source buffer.");
                                                videoReady = true;
                                                checkIfInitialized.call(self, videoReady, audioReady, initialize);
                                            }
                                        );
                                    } else {
                                        self.debug.log("No video data.");
                                        videoReady = true;
                                        checkIfInitialized.call(self, videoReady, audioReady, initialize);
                                    }
                                    
                                    return self.manifestExt.getAudioDatas(manifest);
                                }
                            ).then(
                                function (audioDatas) {
                                    if (audioDatas !== null && audioDatas.length > 0) {
                                        self.debug.log("Have audio streams: " + audioDatas.length);
                                        self.manifestExt.getPrimaryAudioData(manifest).then(
                                            function (primaryAudioData) {
                                                self.manifestExt.getCodec(primaryAudioData).then(
                                                    function (codec) {
                                                        self.debug.log("Audio codec: " + codec);
                                                        if (!self.capabilities.supportsCodec(self.videoModel.getElement(), codec)) {
                                                            self.debug.log("Codec (" + codec + ") is not supported.");
                                                            alert("Codec (" + codec + ") is not supported.");
                                                            return Q.when(null);
                                                        } else {
                                                            return self.sourceBufferExt.createSourceBuffer(mediaSource, codec);
                                                        }
                                                    }
                                                ).then(
                                                    function (buffer) {
                                                        if (buffer === null) {
                                                            self.debug.log("No buffer was created, skipping audio stream.");
                                                        } else {
                                                            // TODO : How to tell index handler live/duration?
                                                            // TODO : Pass to controller and then pass to each method on handler?
                                                            
                                                            audioController = self.system.getObject("bufferController");
                                                            audioController.setType("audio");
                                                            audioController.setData(primaryAudioData);
                                                            audioController.setBuffer(buffer);
                                                            audioController.setMinBufferTime(minBufferTime);
                                                            self.debug.log("Audio is ready!");
                                                        }
                                                        
                                                        audioReady = true;
                                                        checkIfInitialized.call(self, videoReady, audioReady, initialize);
                                                    },
                                                    function (error) {
                                                        alert("Error creating source buffer.");
                                                        audioReady = true;
                                                        checkIfInitialized.call(self, videoReady, audioReady, initialize);
                                                    }
                                                );
                                            }
                                        );
                                    } else {
                                        self.debug.log("No audio streams.");
                                        audioReady = true;
                                        checkIfInitialized.call(self, videoReady, audioReady, initialize);
                                    }
                                }
                            );
                        }
                    );
                }
            );
            
            return initialize.promise;
        },
        
        initializePlayback = function () {
            var self = this,
                initialize = Q.defer();
            
            self.debug.log("Getting ready for playback...");
            
            self.manifestExt.getDuration(manifest).then(
                function (duration) {
                    self.debug.log("Setting duration: " + duration);
                    return self.mediaSourceExt.setDuration(mediaSource, duration);
                }
            ).then(
                function (value) {
                    self.debug.log("Duration successfully set.");
                    initialized = true;
                    initialize.resolve(true);
                }
            );
            
            return initialize.promise;
        },
        
        onPlay = function (e) {
            this.debug.log("Got play event.");
    
            if (!initialized) {
                return;
            }

            this.debug.log("Starting playback.");

            if (videoController) {
                videoController.start();
            }
            if (audioController) {
                audioController.start();
            }
        },
    
        onPause = function (e) {
            this.debug.log("Got pause event.");
    
            if (videoController) {
                videoController.stop();
            }
            if (audioController) {
                audioController.stop();
            }
        },
    
        onSeeking = function (e) {
            this.debug.log("Got seeking event.");

            if (videoController) {
                videoController.seek(this.videoModel.getCurrentTime());
            }
            if (audioController) {
                audioController.seek(this.videoModel.getCurrentTime());
            }
        },
    
        onProgress = function (e) {
            this.debug.log("Got timeupdate event.");
        };
    
    return {
        system: undefined,
        videoModel: undefined,
        manifestLoader: undefined,
        mediaSourceExt: undefined,
        sourceBufferExt: undefined,
        bufferExt: undefined,
        manifestExt: undefined,
        fragmentController: undefined,
        abrController: undefined,
        capabilities: undefined,
        debug: undefined,
        
        setup: function () {
            this.videoModel.listen("play", onPlay.bind(this));
            this.videoModel.listen("pause", onPause.bind(this));
            this.videoModel.listen("seeking", onSeeking.bind(this));
            this.videoModel.listen("timeupdate", onProgress.bind(this));
        },
        
        setAutoPlay: function(value) {
            autoPlay = value;
        },
        
        getAutoPlay: function() {
            return autoPlay;
        },
        
        load: function (url) {
            var self = this;
            
            self.debug.log("Stream start loading.");
            
            self.manifestLoader.load(url).then(
                function (manifestResult) {
                    manifest = manifestResult;
                    self.system.mapValue("manifest", manifest);
                    self.debug.log("Manifest has loaded.");
                    self.debug.log(manifest);
                    return self.mediaSourceExt.createMediaSource();
                }
            ).then(
                function (mediaSourceResult) {
                    mediaSource = mediaSourceResult;
                    self.debug.log("MediaSource created.");
                    return setUpMediaSource.call(self);
                }
            ).then(
                function (result) {
                    self.debug.log("MediaSource set up.");
                    return initializeMediaSource.call(self);
                }
            ).then(
                function (result) {
                    self.debug.log("Start initializing playback.");
                    return initializePlayback.call(self);
                }
            ).then(
                function (done) {
                    self.debug.log("Playback initialized!");
                    self.manifestExt.getIsLive(manifest).then(
                        function (isLive) {
                            if (isLive) {
                                var now = new Date(),
                                    start = self.manifest.availabilityStartTime,
                                    liveOffset = (now.getTime() - start.getTime()) / 1000;
                                self.debug.log("Got live content.  Starting playback at offset: " + liveOffset);
                                seek(liveOffset);
                            } else {
                                self.debug.log("Got VOD content.  Starting playback.");
                                play.call(self);
                            }
                        }
                    );
                }
            );
        },
        
        play: play,
        seek: seek,
        pause: pause
    };
};

MediaPlayer.dependencies.Stream.prototype = {
    constructor: MediaPlayer.dependencies.Stream
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.dependencies.BufferController = function () {
    "use strict";
    
    var VALIDATE_DELAY = 1000,
        WAITING = "WAITING",
        READY = "READY",
        SEEKING = "SEEKING",
        VALIDATING = "VALIDATING",
        LOADING = "LOADING",
        state = WAITING,
        initialPlayback = true,
        seeking = false,
        seekTarget = -1,
        qualityChanged = false,
        lastQuality = -1,
        timer = null,
        
        type,
        data,
        buffer,
        minBufferTime,
        
        doStart = function () {
            if (timer !== null) {
                return;
            }
            
            this.debug.log("BufferController start.");
            state = READY;
            timer = setInterval(onTimer.bind(this), VALIDATE_DELAY, this);
        },
        
        doSeek = function (time) {
            this.debug.log("BufferController seek.");
            seeking = true;
            seekTarget = time;
            
            if (timer === null) {
                doStart();
            }
        },
        
        doStop = function () {
            this.debug.log("BufferController stop.");
            state = WAITING;
            clearInterval(timer);
            timer = null;
        },
        
        onBytesLoaded = function (response) {
            var self = this;
            
            self.debug.log("Bytes finished loading.");
            
            self.fragmentController.process(response.data).then(
                function (data) {
                    if (data !== null) {
                        self.debug.log("Push (" + type + ") bytes: " + data.byteLength);
                        self.debug.log(buffer);
                        self.sourceBufferExt.append(buffer, data);
                    } else {
                        self.debug.log("No bytes to push.");
                    }
                    
                    if (state === LOADING) {
                        state = READY;
                    }
                }
            );
        },
        
        onBytesError = function (error) {
            if (state === LOADING) {
                state = READY;
            }
            
            alert("Error loading fragment.");
            throw(error);
        },
        
        signalStreamComplete = function () {
            doStop.call(this);
        },
        
        loadInitialization = function (qualityChanged, quality) {
            if (initialPlayback) {
                this.debug.log("Marking a special seek for initial playback.");
                seeking = true;
                seekTarget = this.videoModel.getCurrentTime();
                initialPlayback = false;
            }
            
            if (qualityChanged || seeking) {
                return this.indexHandler.getInitRequest(quality, data);
            } else {
                return Q.when(null);
            }
        },
        
        loadNextFragment = function (quality) {
            var promise;
            
            if (seeking) {
                this.debug.log("Loading the fragment for time: " + seekTarget);
                promise = this.indexHandler.getSegmentRequestForTime(seekTarget, quality, data);
                seeking = false;
                seekTarget = -1;
            } else {
                this.debug.log("Loading the next fragment.");
                promise = this.indexHandler.getNextSegmentRequest(quality, data);
            }
            
            return promise;
        },
        
        validate = function () {
            this.debug.log("BufferController.validate() | state: " + state);
            if (state === READY) {
                state = VALIDATING;

                var bufferLength,
                    metrics, // TODO - Get metrics first.
                    newQuality,
                    self = this;
                
                self.sourceBufferExt.getBufferLength(buffer, self.videoModel.getCurrentTime()).then(
                    function (length) {
                        self.debug.log("Current buffer length: " + length);
                        bufferLength = length;
                        return self.bufferExt.shouldBufferMore(length, minBufferTime, metrics);
                    }
                ).then(
                    function (shouldBuffer) {
                        self.debug.log("Deciding to buffer more: " + shouldBuffer);
                        if (shouldBuffer) {
                            self.abrController.getPlaybackQuality(metrics, data).then(
                                function (quality) {
                                    self.debug.log("Playback quality: " + quality);
                                    self.debug.log("Populate buffers.");
                                    
                                    newQuality = quality;
                                    qualityChanged = (quality !== lastQuality);
                                    self.debug.log(qualityChanged ? ("Quality changed to: " + quality) : "Quality didn't change.");
                                    
                                    return loadInitialization.call(self, qualityChanged, quality);
                                }
                            ).then(
                                function (request) {
                                    if (request !== null) {
                                        self.debug.log("Loading initialization: " + request.url);
                                        self.fragmentLoader.load(request).then(onBytesLoaded.bind(self), onBytesError.bind(self));
                                    }
                                    
                                    return loadNextFragment.call(self, newQuality);
                                }
                            ).then(
                                function (request) {
                                    if (request !== null) {
                                        switch (request.action) {
                                            case "stall":
                                                self.debug.log("Stream is stalling.");
                                                break;
                        
                                            case "complete":
                                                self.debug.log("Stream is complete.");
                                                signalStreamComplete.call(self);
                                                break;
                        
                                            case "download":
                                                self.debug.log("Loading a segment: " + request.url);
                                                state = LOADING;
                                                self.fragmentLoader.load(request).then(onBytesLoaded.bind(self), onBytesError.bind(self));
                                                break;
                        
                                            default:
                                                self.debug.log("Unknown request action.");
                                        }
                                    }
                                    
                                    lastQuality = newQuality;
                                    
                                    if (state === VALIDATING) {
                                        state = READY;
                                    }
                                }
                            );
                        } else {
                            if (state === VALIDATING) {
                                state = READY;
                            }
                        }
                    }
                );
            }
        },
        
        onTimer = function () {
            validate.call(this);
        };

    return {
        videoModel: undefined,
        manifestExt: undefined,
        manifest: undefined,
        bufferExt: undefined,
        sourceBufferExt: undefined,
        fragmentController: undefined,
        abrController: undefined,
        fragmentLoader: undefined,
        indexHandler: undefined,
        debug: undefined,
        
        setup: function () {
            var self = this;
            
            self.manifestExt.getIsLive(self.manifest).then(
                function (isLive) {
                    self.indexHandler.setIsLive(isLive);
                }
            );
            
            self.manifestExt.getDuration(self.manifest).then(
                function (duration) {
                    self.indexHandler.setDuration(duration);
                }
            );
        },

        getType: function () {
            return type;
        },
        setType: function (value) {
            type = value;
        },
        
        getData: function () {
            return data;
        },
        setData: function (value) {
            data = value;
        },
        
        getBuffer: function () {
            return buffer;
        },
        setBuffer: function (value) {
            buffer = value;
        },
        
        getMinBufferTime: function () {
            return minBufferTime;
        },
        setMinBufferTime: function (value) {
            minBufferTime = value;
        },
        
        start: doStart,
        seek: doSeek,
        stop: doStop
    };
};

MediaPlayer.dependencies.BufferController.prototype = {
    constructor: MediaPlayer.dependencies.BufferController
};
﻿

/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.rules.BandwidthRule = function () {
    "use strict";
    
    var rules;
    
    return {
        debug: undefined,
        
        checkIndex: function (metrics) {
            if (!metrics) {
                return -1;
            }
            
            var newIdx = -1,
                downloadRatio = metrics.lastFragmentDuration / metrics.lastFragmentDownloadTime,
                switchRatio;

            this.debug.log("Check bandwidth rule.");
            this.debug.log("Current quality index: " + metrics.bitrateIndex);
            this.debug.log("Download ratio: " + downloadRatio);

            if (isNaN(downloadRatio)) {
                this.debug.log("Invalid ratio, bail!");
                newIdx = -1;
            } else if (downloadRatio < 1.0) {
                this.debug.log("Download ratio is poor.");
                if (metrics.bitrateIndex > 0) {
                    this.debug.log("We are not at the lowest bitrate, so switch down.");
                    
                    switchRatio = metrics.getBitrateForIndex(metrics.bitrateIndex - 1) / metrics.getBitrateForIndex(metrics.bitrateIndex);
                    this.debug.log("Switch ratio: " + switchRatio);
                    
                    if (downloadRatio < switchRatio) {
                        this.debug.log("Things must be going pretty bad, switch all the way down.");
                        newIdx = 0;
                    } else {
                        this.debug.log("Things could be better, so just switch down one index.");
                        newIdx = metrics.bitrateIndex - 1;
                    }
                }
            } else {
                this.debug.log("Download ratio is good.");
                if (metrics.bitrateIndex < metrics.maxBitrateIndex) {
                    this.debug.log("We are not at the highest bitrate, so switch up.");
                    
                    switchRatio = metrics.getBitrateForIndex(metrics.bitrateIndex + 1) / metrics.getBitrateForIndex(metrics.bitrateIndex);
                    this.debug.log("Switch ratio: " + switchRatio);
                    
                    if (downloadRatio >= switchRatio) {
                        if (downloadRatio > 1000.0) {
                            this.debug.log("Tons of bandwidth available, go all the way up.");
                            newIdx = metrics.maxBitrateIndex - 1;
                        }
                        else if (downloadRatio > 100.0) {
                            this.debug.log("Just enough bandwidth available, switch up one.");
                            newIdx = metrics.bitrateIndex + 1;
                        }
                        else {
                            this.debug.log("Not exactly sure where to go, so do some math.");
                            while ((newIdx += 1) < metrics.maxBitrateIndex + 1) {
                                switchRatio = metrics.getBitrateForIndex(newIdx) / metrics.getBitrateForIndex(metrics.bitrateIndex);
                                if (downloadRatio < switchRatio) {
                                    break;
                                }
                            }
                            this.debug.log("Calculated ideal new quality index is: " + newIdx);
                            newIdx -= 1;
                        }
                    } else {
                        this.debug.log("Not enough bandwidth to switch up.");
                    }
                }
            }

            this.debug.log("Proposed index: " + newIdx);
            return newIdx;
        }
    };
};

MediaPlayer.rules.BandwidthRule.prototype = {
    constructor: MediaPlayer.rules.BandwidthRule
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
MediaPlayer.rules.BaseRulesCollection = function () {
    "use strict";
    
    var rules;
    
    return {
        bandwidthRule: undefined,
        
        setup: function () {
            rules = [this.bandwidthRule];
        },
        
        getRules: function () {
            return rules;
        }
    };
};

MediaPlayer.rules.BaseRulesCollection.prototype = {
    constructor: MediaPlayer.rules.BaseRulesCollection
};
﻿/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
Dash = (function () {
    "use strict";
    
    return {
        modules: {},
        dependencies: {},
        vo: {},
        di: {}
    };
}());
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
Dash.di.DashContext = function () {
    return {
        system : undefined,
        setup : function() {
            Dash.di.DashContext.prototype.setup.call(this);
            
            this.system.mapClass('parser', Dash.dependencies.DashParser);
            this.system.mapClass('indexHandler', Dash.dependencies.DashHandler);
            this.system.mapClass('baseURLExt', Dash.dependencies.BaseURLExtensions);
            this.system.mapSingleton('manifestExt', Dash.dependencies.DashManifestExtensions);
        }
    };
};

Dash.di.DashContext.prototype = new MediaPlayer.di.Context();
Dash.di.DashContext.prototype.constructor = Dash.di.DashContext;
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
Dash.vo.Segment = function () {
    "use strict";
    this.indexRange = null;
    this.index = null;
    this.mediaRange = null;
    this.media = null;
    this.duration = NaN;
    this.startTime = NaN;
    this.timescale = NaN;
};

Dash.vo.Segment.prototype = {
    constructor: Dash.vo.Segment
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
Dash.dependencies.DashParser = function () {
    "use strict";
    
    var durationRegex = /PT(([0-9]*)H)?(([0-9]*)M)?(([0-9.]*)S)?/,
        datetimeRegex = /^(\d{4}\-\d\d\-\d\d([tT][\d:\.]*)?)([zZ]|([+\-])(\d\d):(\d\d))?$/,
        numericRegex = /^[-+]?[0-9]+[.]?[0-9]*([eE][-+]?[0-9]+)?$/,
        matchers = [
            {
                type: "duration",
                test: function (str) {
                    return durationRegex.test(str);
                },
                converter: function (str) {
                    var match = durationRegex.exec(str);
                    return (parseFloat(match[2] || 0) * 3600 +
                            parseFloat(match[4] || 0) * 60 +
                            parseFloat(match[6] || 0));
                }
            },
            {
                type: "datetime",
                test: function (str) {
                    return datetimeRegex.test(str);
                },
                converter: function (str) {
                    return new Date(str);
                }
            },
            {
                type: "numeric",
                test: function (str) {
                    return numericRegex.test(str);
                },
                converter: function (str) {
                    return parseFloat(str);
                }
            }
        ],
        
        getCommonValuesMap = function () {
            var adaptationSet,
                representation,
                subRepresentation,
                common;

            common = [
                      {
                          name: 'profiles',
                          merge: false
                      },
                      {
                          name: 'width',
                          merge: false
                      },
                      {
                          name: 'height',
                          merge: false
                      },
                      {
                          name: 'sar',
                          merge: false
                      },
                      {
                          name: 'frameRate',
                          merge: false
                      },
                      {
                          name: 'audioSamplingRate',
                          merge: false
                      },
                      {
                          name: 'mimeType',
                          merge: false
                      },
                      {
                          name: 'segmentProfiles',
                          merge: false
                      },
                      {
                          name: 'codecs',
                          merge: false
                      },
                      {
                          name: 'maximumSAPPeriod',
                          merge: false
                      },
                      {
                          name: 'startsWithSap',
                          merge: false
                      },
                      {
                          name: 'maxPlayoutRate',
                          merge: false
                      },
                      {
                          name: 'codingDependency',
                          merge: false
                      },
                      {
                          name: 'scanType',
                          merge: false
                      },
                      {
                          name: 'FramePacking',
                          merge: true
                      },
                      {
                          name: 'AudioChannelConfiguration',
                          merge: true
                      },
                      {
                          name: 'ContentProtection',
                          merge: true
                      }
                     ];
            
            adaptationSet = {};
            adaptationSet.name = "AdaptationSet";
            adaptationSet.isRoot = false;
            adaptationSet.isArray = true;
            adaptationSet.parent = null;
            adaptationSet.children = [];
            adaptationSet.properties = common;
            
                representation = {};
                representation.name = "Representation";
                representation.isRoot = false;
                representation.isArray = true;
                representation.parent = adaptationSet;
                representation.children = [];
                representation.properties = common;
                adaptationSet.children.push(representation);
            
                    subRepresentation = {};
                    subRepresentation.name = "SubRepresentation";
                    subRepresentation.isRoot = false;
                    subRepresentation.isArray = true;
                    subRepresentation.parent = representation;
                    subRepresentation.children = [];
                    subRepresentation.properties = common;
                    representation.children.push(subRepresentation);

            return adaptationSet;
        },
        
        getSegmentValuesMap = function () {
            var period,
                adaptationSet,
                representation,
                common;

            common = [
                      {
                          name: 'SegmentBase',
                          merge: true
                      },
                      {
                          name: 'SegmentTemplate',
                          merge: true
                      },
                      {
                          name: 'SegmentList',
                          merge: true
                      }
                     ];
            
            period = {};
            period.name = "Period";
            period.isRoot = false;
            period.isArray = true;
            period.parent = null;
            period.children = [];
            period.properties = common;
            
                adaptationSet = {};
                adaptationSet.name = "AdaptationSet";
                adaptationSet.isRoot = false;
                adaptationSet.isArray = true;
                adaptationSet.parent = period;
                adaptationSet.children = [];
                adaptationSet.properties = common;
                period.children.push(adaptationSet);
            
                    representation = {};
                    representation.name = "Representation";
                    representation.isRoot = false;
                    representation.isArray = true;
                    representation.parent = adaptationSet;
                    representation.children = [];
                    representation.properties = common;
                    adaptationSet.children.push(representation);
            
            return period;
        },
        
        getBaseUrlValuesMap = function () {
            var mpd,
                period,
                adaptationSet,
                representation,
                common;

            common = [
                      {
                          name: 'BaseURL',
                          merge: true,
                          mergeFunction: function (parentValue, childValue) {
                              // child is absolute, don't merge
                              if (childValue.indexOf("http://") === 0) {
                                  return childValue;
                              } else {
                                  return parentValue + childValue;
                              }
                          }
                      }
                     ];
            
            mpd = {};
            mpd.name = "mpd";
            mpd.isRoot = true;
            mpd.isArray = true;
            mpd.parent = null;
            mpd.children = [];
            mpd.properties = common;
            
                period = {};
                period.name = "Period";
                period.isRoot = false;
                period.isArray = true;
                period.parent = null;
                period.children = [];
                period.properties = common;
                mpd.children.push(period);
                
                    adaptationSet = {};
                    adaptationSet.name = "AdaptationSet";
                    adaptationSet.isRoot = false;
                    adaptationSet.isArray = true;
                    adaptationSet.parent = period;
                    adaptationSet.children = [];
                    adaptationSet.properties = common;
                    period.children.push(adaptationSet);
                
                        representation = {};
                        representation.name = "Representation";
                        representation.isRoot = false;
                        representation.isArray = true;
                        representation.parent = adaptationSet;
                        representation.children = [];
                        representation.properties = common;
                        adaptationSet.children.push(representation);
            
            return mpd;
        },
        
        getDashMap = function () {
            var result = [];
            
            result.push(getCommonValuesMap());
            result.push(getSegmentValuesMap());
            result.push(getBaseUrlValuesMap());
            
            return result;
        },
        
        internalParse = function (data, baseUrl) {
            this.debug.log("Doing parse.");
            
            var manifest,
                converter = new X2JS(matchers, '', true),
                iron = new ObjectIron(getDashMap());
            
            this.debug.log("Converting from XML.");
            manifest = converter.xml_str2json(data);
            
            if (!manifest.hasOwnProperty("BaseURL")) {
                this.debug.log("Setting baseURL: " + baseUrl);
                manifest.BaseURL = baseUrl;
            }
            if (manifest.BaseURL.indexOf("http") !== 0) {
                manifest.BaseURL = baseUrl + manifest.BaseURL;
            }
            
            this.debug.log("Flatten manifest properties.");
            iron.run(manifest);
            
            this.debug.log("Parsing complete.");
            return Q.when(manifest);
        };
    
    return {
        debug: undefined,
        parse: internalParse
    };
};

Dash.dependencies.DashParser.prototype = {
    constructor: Dash.dependencies.DashParser
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
Dash.dependencies.DashHandler = function ()
{
    "use strict";
    
    var index = -1,

        isLive,
        duration,

        getRepresentationForQuality = function (quality, data) {
            var representation = null;
            if (data && data.Representation_asArray && data.Representation_asArray.length > 0) {
                representation = data.Representation_asArray[quality];
            }
            return representation;
        },
        
        replaceNumberForTemplate = function (url, value) {
            var v = value.toString();
            return url.split("$Number$").join(v);
        },
        
        replaceTimeForTemplate = function (url, value) {
            var v = value.toString();
            return url.split("$Time$").join(v);
        },
        
        replaceBandwidthForTemplate = function (url, value) {
            var v = value.toString();
            return url.split("$Bandwidth$").join(v);
        },
        
        replaceIDForTemplate = function (url, value) {
            var v = value.toString();
            return url.split("$RepresentationID$").join(v);
        },
        
        getRequestUrl = function (destination, baseURL) {
            var url;
            
            if (destination === baseURL) {
                url = destination;
            } else if (destination.indexOf("http://") !== -1) {
                url = destination;
            } else {
                url = baseURL + destination;
            }
            
            return url;
        },
        
        getInit = function (quality, data) {
            var deferred = Q.defer(),
                representation = getRepresentationForQuality(quality, data),
                request = null,
                initialization = null,
                url = null,
                range = null,
                self = this;
                
            self.debug.log("Getting the initialization request.");
            
            if (representation.hasOwnProperty("SegmentTemplate")) {
                if (representation.SegmentTemplate.hasOwnProperty("initialization")) {
                    initialization = representation.SegmentTemplate.initialization;
                    initialization = replaceBandwidthForTemplate(initialization, representation.bandwidth);
                    initialization = replaceIDForTemplate(initialization, representation.id);
                }
            }
            else if (representation.hasOwnProperty("SegmentList") &&
                     representation.SegmentList.hasOwnProperty("Initialization") &&
                     representation.SegmentList.Initialization.hasOwnProperty("sourceURL")) {
                initialization = representation.SegmentList.Initialization.sourceURL;     
            }
            else if (representation.hasOwnProperty("SegmentBase") &&
                     representation.SegmentBase.hasOwnProperty("Initialization") &&
                     representation.SegmentBase.Initialization.hasOwnProperty("range")) {
                initialization = representation.BaseURL;
                range = representation.SegmentBase.Initialization.range;
            }
            else {
                // Go out and find the initialization.
                url = representation.BaseURL;
                self.baseURLExt.loadInitialization(url).then(
                    function (theRange) {
                        self.debug.log("Got an initialization.");
                        request = new MediaPlayer.vo.SegmentRequest();
                        request.url = getRequestUrl(url, representation.BaseURL);
                        request.range = theRange;
                        deferred.resolve(request);
                    }
                );
            }
            
            if (initialization && initialization.length > 0) {
                self.debug.log("Got an initialization.");
                request = new MediaPlayer.vo.SegmentRequest();
                request.url = getRequestUrl(initialization, representation.BaseURL);
                request.range = range;
                deferred.resolve(request);
            }
            
            return deferred.promise;
        },

        isMediaFinished = function (representation) { // TODO
            var fragLength,
                isFinished = false;

            this.debug.log("Checking for stream end...");
            if (isLive) {
                this.debug.log("Live never ends! (TODO)");
                // TODO : Check the contents of the last box to signal end.
                isFinished = false;
            }
            else if (representation.hasOwnProperty("segments") && representation.segments !== null) {
                this.debug.log("Segments: " + index + " / " + representation.segments.length);
                isFinished = (index >= representation.segments.length);
            }
            else if (representation.hasOwnProperty("SegmentTemplate") && !representation.SegmentTemplate.hasOwnProperty("SegmentTimeline")) {
                var fDuration,
                    fTimescale = 1,
                    fLength,
                    sDuration = Math.floor(duration); // Disregard fractional seconds.  TODO : Is this ok?  The logic breaks if we don't do this...
                
                if (representation.SegmentTemplate.hasOwnProperty("duration")) {
                    fDuration = representation.SegmentTemplate.duration;
                    
                    if (representation.SegmentTemplate.hasOwnProperty("timescale")) {
                        fTimescale = representation.SegmentTemplate.timescale;
                    }
                    
                    fLength = (fDuration / fTimescale);
                    this.debug.log("SegmentTemplate: " + fLength + " * " + index + " = " + (fLength * index) + " / " + sDuration);
                    isFinished = ((fLength * index) >= sDuration);
                }
            }

            return Q.when(isFinished);
        },
        
        getSegmentsFromTimeline = function (template, timeline) {
            var segments = [],
                fragments,
                frag,
                i,
                len,
                j,
                repeat,
                seg,
                time = 0,
                count = 0,
                url,
                tIdx;
            
            fragments = timeline.S_asArray;
            for (i = 0, len = fragments.length; i < len; i += 1) {
                frag = fragments[i];
                repeat = 0;
                if (frag.hasOwnProperty("r")) {
                    repeat = frag.r;
                }
                
                for (j = 0; j <= repeat; j += 1) {
                    seg = new Dash.vo.Segment();
                    
                    seg.timescale = template.timescale;
                    if (frag.hasOwnProperty("t")) {
                        seg.startTime = frag.t;
                        time = frag.t;
                    } else {
                        seg.startTime = time;
                    }
                    
                    seg.duration = frag.d;
                    
                    if (template.hasOwnProperty("startNumber")) {
                        tIdx = template.startNumber + count;
                    } else {
                        tIdx = count;
                    }
                    
                    url = template.media;
                    url = replaceNumberForTemplate(url, tIdx);
                    url = replaceTimeForTemplate(url, seg.startTime);
                    seg.media = url;
                    
                    segments.push(seg);

                    time += seg.duration;
                    count += 1;
                }
            }
            
            return Q.when(segments);
        },
        
        getSegmentsFromList = function (list) {
            var segments = [],
                i,
                len,
                seg,
                s;

            for (i = 0, len = list.SegmentURL_asArray.length; i < len; i += 1) {
                s = list.SegmentURL_asArray[i];
                
                seg = new Dash.vo.Segment();
                seg.media = s.media;
                seg.mediaRange = s.mediaRange;
                seg.index = s.index;
                seg.indexRange = s.indexRange;
                
                seg.timescale = list.timescale;
                seg.duration = list.duration;
                seg.startTime = i * list.duration;
                
                segments.push(seg);
            }
            
            return Q.when(segments);
        },
        
        getSegmentsFromSource = function (representation) {
            var url = representation.BaseURL,
                range = null;
            
            if (representation.hasOwnProperty("SegmentBase")) {
                if (representation.SegmentBase.hasOwnProperty("indexRange")) {
                    range = representation.SegmentBase.indexRange;
                }
            }
            
            return this.baseURLExt.loadSegments(url, range);
        },
        
        getSegments = function (representation) {
            // We don't need a list of segments in this case.
            if (representation.hasOwnProperty("SegmentTemplate") && !representation.SegmentTemplate.hasOwnProperty("SegmentTimeline")) {
                return Q.when(null);
            }
            else {
                // Already figure out the segments.
                if (representation.hasOwnProperty("segments") && representation.segments !== null) {
                    return Q.when(representation.segments);
                }
                else {
                    if (representation.hasOwnProperty("SegmentTemplate") && representation.SegmentTemplate.hasOwnProperty("SegmentTimeline")) {
                        return getSegmentsFromTimeline.call(this, representation.SegmentTemplate, representation.SegmentTemplate.SegmentTimeline);
                    }
                    else if (representation.hasOwnProperty("SegmentList")) {
                        return getSegmentsFromList.call(this, representation.SegmentList);
                    }
                    else {
                        return getSegmentsFromSource.call(this, representation);
                    }
                }
            }
        },
        
        getIndexForSegments = function (time, segments) {
            var idx = -1,
                frag,
                ft;
            
            if (segments) {
                idx = 0;
                ft = 0;
                frag = null;

                while (ft <= time && idx < segments.length) {
                    idx += 1;
                    frag = segments[idx];
                    ft += frag.duration / frag.timescale;
                }
                idx -= 1;
            }
            
            return Q.when(idx);
        },
        
        getIndexForTemplate = function (time, template) {
            var idx = -1,
                fDuration,
                fTimescale = 1,
                dur;
                
            if (template.hasOwnProperty("duration")) {
                fDuration = template.duration;
            } else {
                throw "Expected 'duration' attribute on SegmentTemplate!";
            }
            
            // default to 1 if not present
            if (template.hasOwnProperty("timescale")) {
                fTimescale = template.timescale;
            }
            
            dur = (fDuration / fTimescale);
            idx = Math.floor(time / dur);
            
            return Q.when(idx);
        },
        
        getRequestForTemplate = function (index, template, representation) {
            var request = new MediaPlayer.vo.SegmentRequest(),
                url,
                tIdx,
                time;

            if (template.hasOwnProperty("startNumber")) {
                tIdx = template.startNumber + index;
            } else {
                tIdx = index;
            }

            time = (template.duration * index);
            if (template.hasOwnProperty("timescale")) {
                time = time / template.timescale;
            }

            url = template.media;
            
            url = replaceNumberForTemplate(url, tIdx);
            url = replaceTimeForTemplate(url, time);
            url = replaceBandwidthForTemplate(url, representation.bandwidth);
            url = replaceIDForTemplate(url, representation.id);
            
            request.url = getRequestUrl(url, representation.BaseURL);
            request.timescale = template.timescale;
            request.duration = template.duration;
            request.startTime = tIdx * template.duration;
            
            return Q.when(request);
        },
        
        getRequestForSegment = function (index, segment, representation) {
            var request = new MediaPlayer.vo.SegmentRequest(),
                url;
            
            url = getRequestUrl(segment.media, representation.BaseURL);
            url = replaceNumberForTemplate(url, index);
            url = replaceTimeForTemplate(url, segment.startTime);
            url = replaceBandwidthForTemplate(url, representation.bandwidth);
            url = replaceIDForTemplate(url, representation.id);
            
            request.url = url;
            request.range = segment.mediaRange;
            request.startTime = segment.startTime / segment.timescale;
            request.duration = segment.duration / segment.timescale;
            
            return Q.when(request);
        },
        
        getForTime = function (time, quality, data) {
            var deferred,
                representation = getRepresentationForQuality(quality, data),
                request,
                segment,
                usingTemplate = false,
                self = this;
            
            self.debug.log("Getting the request for time: " + time);
            
            deferred = Q.defer();
            
            getSegments.call(self, representation).then(
                function (segments) {
                    self.debug.log("Got segments.");
                    self.debug.log(segments);
                    // There's no segments so we *must* have a SegmentTemplate.
                    if (segments === null) {
                        if (!representation.hasOwnProperty("SegmentTemplate")) {
                            throw "Expected SegmentTemplate!";
                        }
                        usingTemplate = true;
                        self.debug.log("No segments found, so we must be using a SegmentTemplate.");
                        return getIndexForTemplate.call(self, time, representation.SegmentTemplate);
                    } else {
                        self.debug.log("Got a list of segments, so dig deeper.");
                        representation.segments = segments;
                        usingTemplate = false;
                        return getIndexForSegments.call(self, time, segments);
                    }
                }
            ).then(
                function (newIndex) {
                    self.debug.log("Index for time " + time + " is " + newIndex);
                    index = newIndex;
                    
                    return isMediaFinished.call(self, representation);
                }
            ).then(
                function (finished) {
                    self.debug.log("Stream finished? " + finished);
                    if (finished) {
                        request = new MediaPlayer.vo.SegmentRequest();
                        request.action = request.ACTION_COMPLETE;
                        self.debug.log("Signal complete.");
                        self.debug.log(request);
                        deferred.resolve(request);
                    } else {
                        if (usingTemplate) {
                            return getRequestForTemplate.call(self, index, representation.SegmentTemplate, representation);
                        } else {
                            segment = representation.segments[index];
                            return getRequestForSegment.call(self, index, segment, representation);
                        }
                    }
                }
            ).then(
                function (request) {
                    self.debug.log("Got a request.");
                    self.debug.log(request);
                    deferred.resolve(request);
                }
            );
            
            return deferred.promise;
        },

        getNext = function (quality, data) {
            var deferred,
                representation = getRepresentationForQuality(quality, data),
                request,
                segment,
                self = this;

            self.debug.log("Getting the next request.");

            if (index === -1) {
                throw "You must call getSegmentRequestForTime first.";
            }
            
            index += 1;
            self.debug.log("New index: " + index);
            
            deferred = Q.defer();
            
            isMediaFinished.call(self, representation).then(
                function (finished) {
                    self.debug.log("Stream finished? " + finished);
                    if (finished) {
                        request = new MediaPlayer.vo.SegmentRequest();
                        request.action = request.ACTION_COMPLETE;
                        self.debug.log("Signal complete.");
                        self.debug.log(request);
                        deferred.resolve(request);
                    } else {
                        getSegments.call(self, representation).then(
                            function (segments) {
                                self.debug.log("Got segments.");
                                self.debug.log(segments);
                                // There's no segments so we *must* have a SegmentTemplate.
                                if (segments === null) {
                                    if (!representation.hasOwnProperty("SegmentTemplate")) {
                                        throw "Expected SegmentTemplate!";
                                    }
                                    self.debug.log("No segments found, so we must be using a SegmentTemplate.");
                                    return getRequestForTemplate.call(self, index, representation.SegmentTemplate, representation);
                                } else {
                                    representation.segments = segments;
                                    segment = representation.segments[index];
                                    return getRequestForSegment.call(self, index, segment, representation);
                                }
                            }
                        ).then(
                            function (request) {
                                self.debug.log("Got a request.");
                                self.debug.log(request);
                                deferred.resolve(request);
                            }
                        );
                    }
                }
            );
            
            return deferred.promise;
        };
    
    return {
        debug: undefined,
        baseURLExt: undefined,
        
        getIsLive: function () {
            return isLive;
        },
        setIsLive: function (value) {
            isLive = value;
        },
        
        getDuration: function () {
            return duration;
        },
        setDuration: function (value) {
            duration = value;
        },
        
        getInitRequest: getInit,
        getSegmentRequestForTime: getForTime,
        getNextSegmentRequest: getNext
    };
};

Dash.dependencies.DashHandler.prototype = {
    constructor: Dash.dependencies.DashHandler
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use self file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
Dash.dependencies.BaseURLExtensions = function () {
    "use strict";
    
    var url,
        range,
        searching = false,
        bytesLoaded = 0,
        bytesToLoad = 1500,
        
        // From YouTube player.  Reformatted for JSLint.
        parseSIDX = function (ab, ab_first_byte_offset) {
            var d = new DataView(ab),
                pos = 0,
                sidxEnd,
                version,
                timescale,
                earliest_presentation_time,
                first_offset,
                reference_count,
                offset,
                time,
                references,
                i,
                ref_size,
                ref_type,
                ref_dur;

            while (d.getUint32(pos + 4, false) !== 0x73696478) {
                pos += d.getUint32(pos, false);
                if (pos >= ab.byteLength) {
                    throw "Could not find sidx";
                }
            }

            sidxEnd = d.getUint32(pos, false) + pos;
            if (sidxEnd > ab.byteLength) {
                throw "sidx terminates after array buffer";
            }

            version = d.getUint8(pos + 8);
            pos += 12;

            // skipped reference_ID(32)
            timescale = d.getUint32(pos + 4, false);
            pos += 8;

            if (version === 0) {
                earliest_presentation_time = d.getUint32(pos, false);
                first_offset = d.getUint32(pos + 4, false);
                pos += 8;
            } else {
                // TODO(strobe): Overflow checks
                earliest_presentation_time = (d.getUint32(pos, false) << 32) + d.getUint32(pos + 4, false);
                first_offset = (d.getUint32(pos + 8, false) << 32) + d.getUint32(pos + 12, false);
                pos += 16;
            }
            first_offset += sidxEnd + (ab_first_byte_offset || 0);

            // skipped reserved(16)
            reference_count = d.getUint16(pos + 2, false);
            pos += 4;

            offset = first_offset;
            time = earliest_presentation_time;
            references = [];

            for (i = 0; i < reference_count; i += 1) {
                ref_size = d.getUint32(pos, false);
                ref_type = ref_size & 0x80000000;
                //if (ref_type) throw "Unhandled indirect reference";
                ref_size = ref_size & 0x7fffffff;
                ref_dur = d.getUint32(pos + 4, false);
                pos += 12;
                references.push({
                    'size': ref_size,
                    'offset': offset,
                    'duration': ref_dur,
                    'time': time,
                    'timescale': timescale
                });
                offset += ref_size;
                time += ref_dur;
            }

            if (pos !== sidxEnd) {
                throw "Error: final pos " + pos + " differs from SIDX end " + sidxEnd;
            }

            return references;
        },
        
        parseSegments = function (data, media, offset) {
            var parsed,
                segments,
                segment,
                i,
                len,
                start,
                end;

            parsed = parseSIDX.call(this, data, offset);
            segments = [];

            for (i = 0, len = parsed.length; i < len; i += 1) {
                segment = new Dash.vo.Segment();
                segment.duration = parsed[i].duration;
                segment.media = media;
                segment.startTime = parsed[i].time;
                segment.timescale = parsed[i].timescale;
                
                start = parsed[i].offset;
                end = parsed[i].offset + parsed[i].size - 1;
                segment.mediaRange = start + "-" + end;
                
                segments.push(segment);
            }

            this.debug.log("Parsed SIDX box: " + segments.length + " segments.");
            return Q.when(segments);
        },
        
        findInit = function (data) {
            var deferred = Q.defer(),
                start,
                end,
                range,
                d = new DataView(data),
                pos = 0,
                type = "",
                size = 0,
                bytesAvailable,
                i,
                c,
                request,
                self = this;

            self.debug.log("Searching for initialization.");

            while (type !== "moov" && pos < d.byteLength) {
                size = d.getUint32(pos); // subtract 8 for including the size and type
                pos += 4;

                type = "";
                for (i = 0; i < 4; i += 1) {
                    c = d.getInt8(pos);
                    type += String.fromCharCode(c);
                    pos += 1;
                }

                if (type !== "moov") {
                    pos += size - 8;
                }
            }
            
            bytesAvailable = d.byteLength - pos;
            
            if (type !== "moov") {
                // Case 1
                // We didn't download enough bytes to find the moov.
                // TODO : Load more bytes.
                //        Be sure to detect EOF.
                //        Throw error is no moov is found in the entire file.
                //        Protection from loading the entire file?
                self.debug.log("Loading more bytes to find initialization.");
                range.start = 0;
                range.end = bytesLoaded + bytesToLoad;

                request = new XMLHttpRequest();
                
                request.onload = function () {
                    bytesLoaded = range.end;
                    findInit.call(self, request.response).then(
                        function (segments) {
                            deferred.resolve(segments);
                        }
                    );
                };
                
                request.onerror = function () {
                    deferred.reject("Error loading initialization.");
                };
                
                request.responseType = "arraybuffer";
                request.open("GET", url);
                request.setRequestHeader("Range", "bytes=" + range.start + "-" + range.end);
                request.send(null);
            } else {
                // Case 2
                // We have the entire range, so continue.
                start = pos - 8;
                end = start + size - 1;
                range = start + "-" + end;

                self.debug.log("Found the initialization.  Range: " + range);
                deferred.resolve(range);
            }
            
            return deferred.promise;
        },
        
        loadInit = function(media) {
            var deferred = Q.defer(),
                request = new XMLHttpRequest(),
                self = this;
            
            url = media;
            range = {};
            
            self.debug.log("Start searching for initialization.");
            range.start = 0;
            range.end = bytesToLoad;
            
            request.onload = function () {
                bytesLoaded = range.end;
                findInit.call(self, request.response).then(
                    function (range) {
                        deferred.resolve(range);
                    }
                );
            };
            
            request.onerror = function () {
                deferred.reject("Error finding initialization.");
            };
            
            request.responseType = "arraybuffer";
            request.open("GET", url);
            request.setRequestHeader("Range", "bytes=" + range.start + "-" + range.end);
            request.send(null);
            self.debug.log("Perform init search: " + url);
            
            return deferred.promise;
        },
        
        findSIDX = function (data) {
            var deferred = Q.defer(),
                d = new DataView(data),
                pos = 0,
                type = "",
                size = 0,
                bytesAvailable,
                sidxBytes,
                i,
                c,
                self = this;

            self.debug.log("Searching for SIDX box.");
            self.debug.log(bytesLoaded + " bytes loaded.");

            while (type !== "sidx" && pos < d.byteLength) {
                size = d.getUint32(pos); // subtract 8 for including the size and type
                pos += 4;

                type = "";
                for (i = 0; i < 4; i += 1) {
                    c = d.getInt8(pos);
                    type += String.fromCharCode(c);
                    pos += 1;
                }

                if (type !== "sidx") {
                    pos += size - 8;
                }
            }

            bytesAvailable = d.byteLength - pos;

            if (type !== "sidx") {
                // Case 1
                // We didn't download enough bytes to find the sidx.
                // TODO : Load more bytes.
                //        Be sure to detect EOF.
                //        Throw error is no sidx is found in the entire file.
                //        Protection from loading the entire file?
                throw ("Could not find SIDX box!");
            } else if (bytesAvailable < size) {
                // Case 2
                // We don't have the entire box.
                // Increase the number of bytes to read and load again.
                self.debug.log("Found SIDX but we don't have all of it.");

                range.start = 0;
                range.end = bytesLoaded + (size - bytesAvailable);

                var request = new XMLHttpRequest();
                
                request.onload = function () {
                    bytesLoaded = range.end;
                    findSIDX.call(self, request.response).then(
                        function (segments) {
                            deferred.resolve(segments);
                        }
                    );
                };
                
                request.onerror = function () {
                    deferred.reject("Error loading sidx.");
                };
                
                request.responseType = "arraybuffer";
                request.open("GET", url);
                request.setRequestHeader("Range", "bytes=" + range.start + "-" + range.end);
                request.send(null);
            } else {
                // Case 3
                // We have the entire box, so parse it and continue.
                range.start = pos - 8;
                range.end = range.start + size;

                self.debug.log("Found the SIDX box.  Start: " + range.start + " | End: " + range.end);
                sidxBytes = data.slice(range.start, range.end);
                
                parseSegments.call(self, sidxBytes, url, range.start).then(
                    function (segments) {
                        deferred.resolve(segments);
                    }
                );
            }
            
            return deferred.promise;
        },
        
        loadSegments = function(media, theRange) {
            var deferred = Q.defer(),
                request = new XMLHttpRequest(),
                parts,
                self = this;
            
            url = media;
            range = {};
            
            // We might not know exactly where the sidx box is.
            // Load the first n bytes (say 1500) and look for it.
            if (theRange === null) {
                self.debug.log("No known range for SIDX request.");
                searching = true;
                range.start = 0;
                range.end = bytesToLoad;
            } else {
                parts = theRange.split("-");
                range.start = parseFloat(parts[0]);
                range.end = parseFloat(parts[1]);
            }
            
            request.onload = function () {
                // If we didn't know where the SIDX box was, we have to look for it.
                // Iterate over the data checking out the boxes to find it.
                if (searching) {
                    bytesLoaded = range.end;
                    findSIDX.call(self, request.response).then(
                        function (segments) {
                            deferred.resolve(segments);
                        }
                    );
                } else {
                    parseSegments.call(self, request.response, url, range.start).then(
                        function (segments) {
                            deferred.resolve(segments);
                        }
                    );
                }
            };
            
            request.onerror = function () {
                deferred.reject("Error loading sidx.");
            };
            
            request.responseType = "arraybuffer";
            request.open("GET", url);
            request.setRequestHeader("Range", "bytes=" + range.start + "-" + range.end);
            request.send(null);
            self.debug.log("Perform SIDX load: " + url);
            
            return deferred.promise;
        };
    
    return {
        debug: undefined,
        
        loadSegments: loadSegments,
        loadInitialization: loadInit
    };
};

Dash.dependencies.BaseURLExtensions.prototype = {
    constructor: Dash.dependencies.BaseURLExtensions
};
/*
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * copyright Digital Primates 2012
 */
Dash.dependencies.DashManifestExtensions = function () {
    "use strict";
};

Dash.dependencies.DashManifestExtensions.prototype = {
    constructor: Dash.dependencies.DashManifestExtensions,
    
    getIsAudio: function (adaptation) { // TODO : could come from representation or elsewhere too?
            var i,
                len,
                col = adaptation.ContentComponent_asArray,
                representation,
                result = false,
                found = false;

            if (col) {
                for (i = 0, len = col.length; i < len; i += 1) {
                    if (col[i].contentType === "audio") {
                        result = true;
                        found = true;
                    }
                }
            }

            if (adaptation.hasOwnProperty("mimeType")) {
                result = adaptation.mimeType.indexOf("audio") !== -1;
                found = true;
            }
            
            // couldn't find on adaptationset, so check a representation
            if (!found) {
                i = 0;
                len = adaptation.Representation_asArray.length;
                while (!found && i < len) {
                    representation = adaptation.Representation_asArray[i];
                    
                    if (representation.hasOwnProperty("mimeType")) {
                        result = representation.mimeType.indexOf("audio") !== -1;
                        found = true;
                    }
                    
                    i += 1;
                }
            }

            return Q.when(result);
        },

        getIsVideo: function (adaptation) { // TODO : could come from representation or elsewhere too?
            var i,
                len,
                col = adaptation.ContentComponent_asArray,
                representation,
                result = false,
                found = false;

            if (col) {
                for (i = 0, len = col.length; i < len; i += 1) {
                    if (col[i].contentType === "video") {
                        result = true;
                        found = true;
                    }
                }
            }

            if (adaptation.hasOwnProperty("mimeType")) {
                result = adaptation.mimeType.indexOf("video") !== -1;
                found = true;
            }
            
            // couldn't find on adaptationset, so check a representation
            if (!found) {
                i = 0;
                len = adaptation.Representation_asArray.length;
                while (!found && i < len) {
                    representation = adaptation.Representation_asArray[i];
                    
                    if (representation.hasOwnProperty("mimeType")) {
                        result = representation.mimeType.indexOf("video") !== -1;
                        found = true;
                    }
                    
                    i += 1;
                }
            }

            return Q.when(result);
        },

        getIsMain: function (adaptation) {
            // TODO : Check "Role" node.
            // TODO : Use this somewhere.
            return Q.when(false);
        },
        
        getVideoData: function(manifest) {
            //return Q.when(null);
            //------------------------------------
            var adaptations = manifest.Period_asArray[0].AdaptationSet_asArray,
                i,
                len,
                data,
                deferred = Q.defer(),
                funcs = [];

            for (i = 0, len = adaptations.length; i < len; i += 1) {
                funcs.push(this.getIsVideo(adaptations[i]));
            }
            
            Q.all(funcs).then(
                function (results) {
                    var found = false;
                    for (i = 0, len = results.length; i < len; i += 1) {
                        if (results[i] === true) {
                            found = true;
                            deferred.resolve(adaptations[i]);
                        }
                    }
                    if (!found) {
                        deferred.resolve(null);
                    }
                }
            );
            
            return deferred.promise;
        },
        
        getAudioDatas: function(manifest) {
            //return Q.when(null);
            //------------------------------------
            var adaptations = manifest.Period_asArray[0].AdaptationSet_asArray,
                i,
                len,
                data,
                deferred = Q.defer(),
                funcs = [];

            for (i = 0, len = adaptations.length; i < len; i += 1) {
                funcs.push(this.getIsAudio(adaptations[i]));
            }
            
            Q.all(funcs).then(
                function (results) {
                    var datas = [];
                    for (i = 0, len = results.length; i < len; i += 1) {
                        if (results[i] === true) {
                            datas.push(adaptations[i]);
                        }
                    }
                    deferred.resolve(datas);
                }
            );
            
            return deferred.promise;
        },
        
        getPrimaryAudioData: function(manifest) {
            var adaptations = manifest.Period_asArray[0].AdaptationSet_asArray,
                i,
                len,
                data,
                deferred = Q.defer(),
                funcs = [],
                self = this;
            
            this.getAudioDatas(manifest).then(
                function (datas) {
                    if (!datas || datas.length === 0) {
                        deferred.resolve(null);
                    }
                    
                    for (i = 0, len = datas.length; i < len; i += 1) {
                        funcs.push(self.getIsMain(datas[i]));
                    }
                    
                    Q.all(funcs).then(
                        function (results) {
                            var found = false;
                            for (i = 0, len = results.length; i < len; i += 1) {
                                if (results[i] === true) {
                                    found = true;
                                    deferred.resolve(datas[i]);
                                }
                            }
                            if (!found) {
                                deferred.resolve(datas[0]);
                            }
                        }
                    );
                }
            );
            
            return deferred.promise;
        },
        
        getCodec: function(data) {
            var representation = data.Representation_asArray[0],
                codec = (representation.mimeType + ';codecs="' + representation.codecs + '"');
            return Q.when(codec);
        },
        
        getIsLive: function (manifest) {
            return Q.when(manifest.type === "dynamic");
        },

        getIsDVR: function (manifest) {
            var deferred = Q.defer();
            
            this.getIsLive(manifest).then(
                function (isLive) {
                    var containsDVR = !isNaN(manifest.timeShiftBufferDepth);
                    deferred.resolve(isLive && containsDVR);
                }
            );
            
            return deferred.promise;
        },

        getIsOnDemand: function (manifest) {
            var isOnDemand = false;
            
            if (manifest.profiles && manifest.profiles.length > 0) {
                isOnDemand = (manifest.profiles.indexOf("urn:mpeg:dash:profile:isoff-on-demand:201") !== -1);
            }
            
            return Q.when(isOnDemand);
        },
        
        getDuration: function (manifest) {
            var deferred = Q.defer(),
                dur = NaN;
            
            this.getIsLive(manifest).then(
                function (isLive) {
                    if (isLive) {
                        deferred.resolve(Number.POSITIVE_INFINITY);
                    } else {
                        if (manifest.mediaPresentationDuration) {
                            dur = manifest.mediaPresentationDuration;
                        } else if (manifest.availabilityEndTime && manifest.availabilityStartTime) {
                            dur = (manifest.availabilityEndTime.getTime() - manifest.availabilityStartTime.getTime());
                        }
                        deferred.resolve(dur);
                    }
                }
            );
            
            return deferred.promise;
        }
};
/**
 * @author <a href="http://www.creynders.be">Camille Reynders</a>
 */
( function ( scope ) {

    "use strict";

    /**
     * @namespace
     */
    var dijon = {
        /**
         * framework version number
         * @constant
         * @type String
         */
        VERSION:'0.5.3'
    };//dijon


    //======================================//
    // dijon.System
    //======================================//

    /**
     * @class dijon.System
     * @constructor
     */
    dijon.System = function () {
        /** @private */
        this._mappings = {};

        /** @private */
        this._outlets = {};

        /** @private */
        this._handlers = {};

        /**
         * When <code>true</code> injections are made only if an object has a property with the mapped outlet name.<br/>
         * <strong>Set to <code>false</code> at own risk</strong>, may have quite undesired side effects.
         * @example
         * system.strictInjections = true
         * var o = {};
         * system.mapSingleton( 'userModel', UserModel );
         * system.mapOutlet( 'userModel' );
         * system.injectInto( o );
         *
         * //o is unchanged
         *
         * system.strictInjections = false;
         * system.injectInto( o );
         *
         * //o now has a member 'userModel' which holds a reference to the singleton instance
         * //of UserModel
         * @type Boolean
         * @default true
         */
        this.strictInjections = true;

        /**
         * Enables the automatic mapping of outlets for mapped values, singletons and classes
         * When this is true any value, singleton or class that is mapped will automatically be mapped as a global outlet
         * using the value of <code>key</code> as outlet name
         *
         * @example
         * var o = {
         *     userModel : undefined; //inject
         * }
         * system.mapSingleton( 'userModel', UserModel );
         * system.injectInto( o ):
         * //o.userModel now holds a reference to the singleton instance of UserModel
         * @type Boolean
         * @default false
         */
        this.autoMapOutlets = false;

        /**
         * The name of the method that will be called for all instances, right after injection has occured.
         * @type String
         * @default 'setup'
         */
        this.postInjectionHook = 'setup';

    };//dijon.System

    dijon.System.prototype = {

        /**
         * @private
         * @param {Class} clazz
         */
        _createAndSetupInstance:function ( key, Clazz ) {
            var instance = new Clazz();
            this.injectInto( instance, key );
            return instance;
        },

        /**
         * @private
         * @param {String} key
         * @param {Boolean} overrideRules
         * @return {Object}
         */
        _retrieveFromCacheOrCreate:function ( key, overrideRules ) {
            if ( typeof overrideRules === 'undefined' ) {
                overrideRules = false;
            }
            var output;
            if ( this._mappings.hasOwnProperty( key ) ) {
                var config = this._mappings[ key ];
                if ( !overrideRules && config.isSingleton ) {
                    if ( config.object == null ) {
                        config.object = this._createAndSetupInstance( key, config.clazz );
                    }
                    output = config.object;
                } else {
                    if ( config.clazz ) {
                        output = this._createAndSetupInstance( key, config.clazz );
                    } else {
                        //TODO shouldn't this be null
                        output = config.object;
                    }
                }
            } else {
                throw new Error( 1000 );
            }
            return output;
        },


        /**
         * defines <code>outletName</code> as an injection point in <code>targetKey</code>for the object mapped to <code>sourceKey</code>
         * @example
         system.mapSingleton( 'userModel', TestClassA );
         var o = {
         user : undefined //inject
         }
         system.mapOutlet( 'userModel', 'o', 'user' );
         system.mapValue( 'o', o );

         var obj = system.getObject( 'o' );
         * //obj.user holds a reference to the singleton instance of UserModel
         *
         * @example
         system.mapSingleton( 'userModel', TestClassA );
         var o = {
         userModel : undefined //inject
         }
         system.mapOutlet( 'userModel', 'o' );
         system.mapValue( 'o', o );

         var obj = system.getObject( 'o' );
         * //obj.userModel holds a reference to the singleton instance of UserModel
         *
         * @example
         system.mapSingleton( 'userModel', TestClassA );
         system.mapOutlet( 'userModel' );
         var o = {
         userModel : undefined //inject
         }
         system.mapValue( 'o', o );

         var obj = system.getObject( 'o' );
         * //o.userModel holds a reference to the singleton instance of userModel
         *
         * @param {String} sourceKey the key mapped to the object that will be injected
         * @param {String} [targetKey='global'] the key the outlet is assigned to.
         * @param {String} [outletName=sourceKey] the name of the property used as an outlet.<br/>
         * @return {dijon.System}
         * @see dijon.System#unmapOutlet
         */
        mapOutlet:function ( sourceKey, targetKey, outletName ) {
            if ( typeof sourceKey === 'undefined' ) {
                throw new Error( 1010 );
            }
            targetKey = targetKey || "global";
            outletName = outletName || sourceKey;

            if ( !this._outlets.hasOwnProperty( targetKey ) ) {
                this._outlets[ targetKey ] = {};
            }
            this._outlets[ targetKey ][ outletName ] = sourceKey;

            return this;
        },

        /**
         * Retrieve (or create) the object mapped to <code>key</code>
         * @example
         * system.mapValue( 'foo', 'bar' );
         * var b = system.getObject( 'foo' ); //now contains 'bar'
         * @param {Object} key
         * @return {Object}
         */
        getObject:function ( key ) {
            if ( typeof key === 'undefined' ) {
                throw new Error( 1020 );
            }
            return this._retrieveFromCacheOrCreate( key );
        },

        /**
         * Maps <code>useValue</code> to <code>key</code>
         * @example
         * system.mapValue( 'foo', 'bar' );
         * var b = system.getObject( 'foo' ); //now contains 'bar'
         * @param {String} key
         * @param {Object} useValue
         * @return {dijon.System}
         */
        mapValue:function ( key, useValue ) {
            if ( typeof key === 'undefined' ) {
                throw new Error( 1030 );
            }
            this._mappings[ key ] = {
                clazz:null,
                object:useValue,
                isSingleton:true
            };
            if ( this.autoMapOutlets ) {
                this.mapOutlet( key );
            }
            if ( this.hasMapping( key )) {
                this.injectInto( useValue, key );
            }
            return this;
        },

        /**
         * Returns whether the key is mapped to an object
         * @example
         * system.mapValue( 'foo', 'bar' );
         * var isMapped = system.hasMapping( 'foo' );
         * @param {String} key
         * @return {Boolean}
         */
        hasMapping:function ( key ) {
            if ( typeof key === 'undefined' ) {
                throw new Error( 1040 );
            }
            return this._mappings.hasOwnProperty( key );
        },

        /**
         * Maps <code>clazz</code> as a factory to <code>key</code>
         * @example
         * var SomeClass = function(){
         * }
         * system.mapClass( 'o', SomeClass );
         *
         * var s1 = system.getObject( 'o' );
         * var s2 = system.getObject( 'o' );
         *
         * //s1 and s2 reference two different instances of SomeClass
         *
         * @param {String} key
         * @param {Function} clazz
         * @return {dijon.System}
         */
        mapClass:function ( key, clazz ) {
            if ( typeof key === 'undefined' ) {
                throw new Error( 1050 );
            }
            if ( typeof clazz === 'undefined' ) {
                throw new Error( 1051 );
            }
            this._mappings[ key ] = {
                clazz:clazz,
                object:null,
                isSingleton:false
            };
            if ( this.autoMapOutlets ) {
                this.mapOutlet( key );
            }
            return this;
        },

        /**
         * Maps <code>clazz</code> as a singleton factory to <code>key</code>
         * @example
         * var SomeClass = function(){
         * }
         * system.mapSingleton( 'o', SomeClass );
         *
         * var s1 = system.getObject( 'o' );
         * var s2 = system.getObject( 'o' );
         *
         * //s1 and s2 reference the same instance of SomeClass
         *
         * @param {String} key
         * @param {Function} clazz
         * @return {dijon.System}
         */
        mapSingleton:function ( key, clazz ) {
            if ( typeof key === 'undefined' ) {
                throw new Error( 1060 );
            }
            if ( typeof clazz === 'undefined' ) {
                throw new Error( 1061 );
            }
            this._mappings[ key ] = {
                clazz:clazz,
                object:null,
                isSingleton:true
            };
            if ( this.autoMapOutlets ) {
                this.mapOutlet( key );
            }
            return this;
        },

        /**
         * Force instantiation of the class mapped to <code>key</code>, whether it was mapped as a singleton or not.
         * When a value was mapped, the value will be returned.
         * TODO: should this last rule be changed?
         * @example
         var SomeClass = function(){
         }
         system.mapClass( 'o', SomeClass );

         var s1 = system.getObject( 'o' );
         var s2 = system.getObject( 'o' );
         * //s1 and s2 reference different instances of SomeClass
         *
         * @param {String} key
         * @return {Object}
         */
        instantiate:function ( key ) {
            if ( typeof key === 'undefined' ) {
                throw new Error( 1070 );
            }
            return this._retrieveFromCacheOrCreate( key, true );
        },

        /**
         * Perform an injection into an object's mapped outlets, satisfying all it's dependencies
         * @example
         * var UserModel = function(){
         * }
         * system.mapSingleton( 'userModel', UserModel );
         * var SomeClass = function(){
         *      user = undefined; //inject
         * }
         * system.mapSingleton( 'o', SomeClass );
         * system.mapOutlet( 'userModel', 'o', 'user' );
         *
         * var foo = {
         *      user : undefined //inject
         * }
         *
         * system.injectInto( foo, 'o' );
         *
         * //foo.user now holds a reference to the singleton instance of UserModel
         * @param {Object} instance
         * @param {String} [key] use the outlet mappings as defined for <code>key</code>, otherwise only the globally
         * mapped outlets will be used.
         * @return {dijon.System}
         */
        injectInto:function ( instance, key ) {
            if ( typeof instance === 'undefined' ) {
                throw new Error( 1080 );
            }
			if( ( typeof instance === 'object' ) ){
				var o = [];
				if ( this._outlets.hasOwnProperty( 'global' ) ) {
					o.push( this._outlets[ 'global' ] );
				}
				if ( typeof key !== 'undefined' && this._outlets.hasOwnProperty( key ) ) {
					o.push( this._outlets[ key ] );
				}
				for ( var i in o ) {
					var l = o [ i ];
					for ( var outlet in l ) {
						var source = l[ outlet ];
						//must be "in" [!]
						if ( !this.strictInjections || outlet in instance ) {
							instance[ outlet ] = this.getObject( source );
						}
					}
				}
				if ( "setup" in instance ) {
					instance.setup.call( instance );
				}
			}
            return this;
        },

        /**
         * Remove the mapping of <code>key</code> from the system
         * @param {String} key
         * @return {dijon.System}
         */
        unmap:function ( key ) {
            if ( typeof key === 'undefined' ) {
                throw new Error( 1090 );
            }
            delete this._mappings[ key ];

            return this;
        },

        /**
         * removes an injection point mapping for a given object mapped to <code>key</code>
         * @param {String} target
         * @param {String} outlet
         * @return {dijon.System}
         * @see dijon.System#addOutlet
         */
        unmapOutlet:function ( target, outlet ) {
            if ( typeof target === 'undefined' ) {
                throw new Error( 1100 );
            }
            if ( typeof outlet === 'undefined' ) {
                throw new Error( 1101 );
            }
            delete this._outlets[ target ][ outlet ];

            return this;
        },

        /**
         * maps a handler for an event/route.<br/>
         * @example
         var hasExecuted = false;
         var userView = {
         showUserProfile : function(){
         hasExecuted = true;
         }
         }
         system.mapValue( 'userView', userView );
         system.mapHandler( 'user/profile', 'userView', 'showUserProfile' );
         system.notify( 'user/profile' );
         //hasExecuted is true
         * @example
         * var userView = {
         *      showUserProfile : function(){
         *          //do stuff
         *      }
         * }
         * system.mapValue( 'userView', userView );
         * <strong>system.mapHandler( 'showUserProfile', 'userView' );</strong>
         * system.notify( 'showUserProfile' );
         *
         * //userView.showUserProfile is called
         * @example
         * var showUserProfile = function(){
         *          //do stuff
         * }
         * <strong>system.mapHandler( 'user/profile', undefined, showUserProfile );</strong>
         * system.notify( 'user/profile' );
         *
         * //showUserProfile is called
         * @example
         * var userView = {};
         * var showUserProfile = function(){
         *          //do stuff
         * }
         * system.mapValue( 'userView', userView );
         * <strong>system.mapHandler( 'user/profile', 'userView', showUserProfile );</strong>
         * system.notify( 'user/profile' );
         *
         * //showUserProfile is called within the scope of the userView object
         * @example
         * var userView = {
         *      showUserProfile : function(){
         *          //do stuff
         *      }
         * }
         * system.mapValue( 'userView', userView );
         * <strong>system.mapHandler( 'user/profile', 'userView', 'showUserProfile', true );</strong>
         * system.notify( 'user/profile' );
         * system.notify( 'user/profile' );
         * system.notify( 'user/profile' );
         *
         * //userView.showUserProfile is called exactly once [!]
         * @example
         * var userView = {
         *      showUserProfile : function( route ){
         *          //do stuff
         *      }
         * }
         * system.mapValue( 'userView', userView );
         * <strong>system.mapHandler( 'user/profile', 'userView', 'showUserProfile', false, true );</strong>
         * system.notify( 'user/profile' );
         *
         * //userView.showUserProfile is called and the route/eventName is passed to the handler
         * @param {String} eventName/route
         * @param {String} [key=undefined] If <code>key</code> is <code>undefined</code> the handler will be called without
         * scope.
         * @param {String|Function} [handler=eventName] If <code>handler</code> is <code>undefined</code> the value of
         * <code>eventName</code> will be used as the name of the member holding the reference to the to-be-called function.
         * <code>handler</code> accepts either a string, which will be used as the name of the member holding the reference
         * to the to-be-called function, or a direct function reference.
         * @param {Boolean} [oneShot=false] Defines whether the handler should be called exactly once and then automatically
         * unmapped
         * @param {Boolean} [passEvent=false] Defines whether the event object should be passed to the handler or not.
         * @return {dijon.System}
         * @see dijon.System#notify
         * @see dijon.System#unmapHandler
         */
        mapHandler:function ( eventName, key, handler, oneShot, passEvent ) {
            if ( typeof eventName === 'undefined' ) {
                throw new Error( 1110 );
            }
            key = key || 'global';
            handler = handler || eventName;

            if ( typeof oneShot === 'undefined' ) {
                oneShot = false;
            }
            if ( typeof passEvent === 'undefined' ) {
                passEvent = false;
            }
            if ( !this._handlers.hasOwnProperty( eventName ) ) {
                this._handlers[ eventName ] = {};
            }
            if ( !this._handlers[eventName].hasOwnProperty( key ) ) {
                this._handlers[eventName][key] = [];
            }
            this._handlers[ eventName ][ key ].push( {
                handler:handler,
                oneShot:oneShot,
                passEvent:passEvent
            } );

            return this;
        },

        /**
         * Unmaps the handler for a specific event/route.
         * @param {String} eventName Name of the event/route
         * @param {String} [key=undefined] If <code>key</code> is <code>undefined</code> the handler is removed from the
         * global mapping space. (If the same event is mapped globally and specifically for an object, then
         * only the globally mapped one will be removed)
         * @param {String | Function} [handler=eventName]
         * @return {dijon.System}
         * @see dijon.System#mapHandler
         */
        unmapHandler:function ( eventName, key, handler ) {
            if ( typeof eventName === 'undefined' ) {
                throw new Error( 1120 );
            }
            key = key || 'global';
            handler = handler || eventName;

            if ( this._handlers.hasOwnProperty( eventName ) && this._handlers[ eventName ].hasOwnProperty( key ) ) {
                var handlers = this._handlers[ eventName ][ key ];
                for ( var i in handlers ) {
                    var config = handlers[ i ];
                    if ( config.handler === handler ) {
                        handlers.splice( i, 1 );
                        break;
                    }
                }
            }
            return this;
        },

        /**
         * calls all handlers mapped to <code>eventName/route</code>
         * @param {String} eventName/route
         * @return {dijon.System}
         * @see dijon.System#mapHandler
         */
        notify:function ( eventName ) {
            if ( typeof eventName === 'undefined' ) {
                throw new Error( 1130 );
            }
            var argsWithEvent = Array.prototype.slice.call( arguments );
            var argsClean = argsWithEvent.slice( 1 );
            if ( this._handlers.hasOwnProperty( eventName ) ) {
                var handlers = this._handlers[ eventName ];
                for ( var key in handlers ) {
                    var configs = handlers[ key ];
                    var instance;
                    if ( key !== 'global' ) {
                        instance = this.getObject( key );
                    }
                    var toBeDeleted = [];
                    var i, n;
                    for ( i = 0, n = configs.length ; i < n ; i++ ) {
                        var handler;
                        var config = configs[ i ];
                        if ( instance && typeof config.handler === "string" ) {
                            handler = instance[ config.handler ];
                        } else {
                            handler = config.handler;
                        }

                        //see deletion below
                        if ( config.oneShot ) {
                            toBeDeleted.unshift( i );
                        }

                        if ( config.passEvent ) {
                            handler.apply( instance, argsWithEvent );
                        } else {
                            handler.apply( instance, argsClean );
                        }
                    }

                    //items should be deleted in reverse order
                    //either use push above and decrement here
                    //or use unshift above and increment here
                    for ( i = 0, n = toBeDeleted.length ; i < n ; i++ ) {
                        configs.splice( toBeDeleted[ i ], 1 );
                    }
                }
            }

            return this;
        }

    };//dijon.System.prototype

    scope.dijon = dijon;
}( this ));



// vim:ts=4:sts=4:sw=4:
/*!
 *
 * Copyright 2009-2012 Kris Kowal under the terms of the MIT
 * license found at http://github.com/kriskowal/q/raw/master/LICENSE
 *
 * With parts by Tyler Close
 * Copyright 2007-2009 Tyler Close under the terms of the MIT X license found
 * at http://www.opensource.org/licenses/mit-license.html
 * Forked at ref_send.js version: 2009-05-11
 *
 * With parts by Mark Miller
 * Copyright (C) 2011 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

(function (definition) {
    // Turn off strict mode for this function so we can assign to global.Q
    /*jshint strict: false*/

    // This file will function properly as a <script> tag, or a module
    // using CommonJS and NodeJS or RequireJS module formats.  In
    // Common/Node/RequireJS, the module exports the Q API and when
    // executed as a simple <script>, it creates a Q global instead.

    // Montage Require
    if (typeof bootstrap === "function") {
        bootstrap("promise", definition);

    // CommonJS
    } else if (typeof exports === "object") {
        module.exports = definition();

    // RequireJS
    } else if (typeof define === "function") {
        define(definition);

    // SES (Secure EcmaScript)
    } else if (typeof ses !== "undefined") {
        if (!ses.ok()) {
            return;
        } else {
            ses.makeQ = definition;
        }

    // <script>
    } else {
        Q = definition();
    }

})(function () {
"use strict";

// All code after this point will be filtered from stack traces reported
// by Q.
var qStartingLine = captureLine();
var qFileName;

// shims

// used for fallback in "allResolved"
var noop = function () {};

// use the fastest possible means to execute a task in a future turn
// of the event loop.
var nextTick;
if (typeof process !== "undefined") {
    // node
    nextTick = process.nextTick;
} else if (typeof setImmediate === "function") {
    // In IE10, or use https://github.com/NobleJS/setImmediate
    if (typeof window !== "undefined") {
        nextTick = setImmediate.bind(window);
    } else {
        nextTick = setImmediate;
    }
} else {
    (function () {
        // linked list of tasks (single, with head node)
        var head = {task: void 0, next: null}, tail = head,
            maxPendingTicks = 2, pendingTicks = 0, queuedTasks = 0, usedTicks = 0,
            requestTick;

        function onTick() {
            // In case of multiple tasks ensure at least one subsequent tick
            // to handle remaining tasks in case one throws.
            --pendingTicks;

            if (++usedTicks >= maxPendingTicks) {
                // Amortize latency after thrown exceptions.
                usedTicks = 0;
                maxPendingTicks *= 4; // fast grow!
                var expectedTicks = queuedTasks && Math.min(queuedTasks - 1, maxPendingTicks);
                while (pendingTicks < expectedTicks) {
                    ++pendingTicks;
                    requestTick();
                }
            }

            while (queuedTasks) {
                --queuedTasks; // decrement here to ensure it's never negative
                head = head.next;
                var task = head.task;
                head.task = void 0;
                task();
            }

            usedTicks = 0;
        }

        nextTick = function (task) {
            tail = tail.next = {task: task, next: null};
            if (pendingTicks < ++queuedTasks && pendingTicks < maxPendingTicks) {
                ++pendingTicks;
                requestTick();
            }
        };

        if (typeof MessageChannel !== "undefined") {
            // modern browsers
            // http://www.nonblocking.io/2011/06/windownexttick.html
            var channel = new MessageChannel();
            channel.port1.onmessage = onTick;
            requestTick = function () {
                channel.port2.postMessage(0);
            };

        } else {
            // old browsers
            requestTick = function () {
                setTimeout(onTick, 0);
            };
        }
    })();
}

// Attempt to make generics safe in the face of downstream
// modifications.
// There is no situation where this is necessary.
// If you need a security guarantee, these primordials need to be
// deeply frozen anyway, and if you don’t need a security guarantee,
// this is just plain paranoid.
// However, this does have the nice side-effect of reducing the size
// of the code by reducing x.call() to merely x(), eliminating many
// hard-to-minify characters.
// See Mark Miller’s explanation of what this does.
// http://wiki.ecmascript.org/doku.php?id=conventions:safe_meta_programming
function uncurryThis(f) {
    var call = Function.call;
    return function () {
        return call.apply(f, arguments);
    };
}
// This is equivalent, but slower:
// uncurryThis = Function_bind.bind(Function_bind.call);
// http://jsperf.com/uncurrythis

var array_slice = uncurryThis(Array.prototype.slice);

var array_reduce = uncurryThis(
    Array.prototype.reduce || function (callback, basis) {
        var index = 0,
            length = this.length;
        // concerning the initial value, if one is not provided
        if (arguments.length === 1) {
            // seek to the first value in the array, accounting
            // for the possibility that is is a sparse array
            do {
                if (index in this) {
                    basis = this[index++];
                    break;
                }
                if (++index >= length) {
                    throw new TypeError();
                }
            } while (1);
        }
        // reduce
        for (; index < length; index++) {
            // account for the possibility that the array is sparse
            if (index in this) {
                basis = callback(basis, this[index], index);
            }
        }
        return basis;
    }
);

var array_indexOf = uncurryThis(
    Array.prototype.indexOf || function (value) {
        // not a very good shim, but good enough for our one use of it
        for (var i = 0; i < this.length; i++) {
            if (this[i] === value) {
                return i;
            }
        }
        return -1;
    }
);

var array_map = uncurryThis(
    Array.prototype.map || function (callback, thisp) {
        var self = this;
        var collect = [];
        array_reduce(self, function (undefined, value, index) {
            collect.push(callback.call(thisp, value, index, self));
        }, void 0);
        return collect;
    }
);

var object_create = Object.create || function (prototype) {
    function Type() { }
    Type.prototype = prototype;
    return new Type();
};

var object_hasOwnProperty = uncurryThis(Object.prototype.hasOwnProperty);

var object_keys = Object.keys || function (object) {
    var keys = [];
    for (var key in object) {
        if (object_hasOwnProperty(object, key)) {
            keys.push(key);
        }
    }
    return keys;
};

var object_toString = uncurryThis(Object.prototype.toString);

// generator related shims

function isStopIteration(exception) {
    return (
        object_toString(exception) === "[object StopIteration]" ||
        exception instanceof QReturnValue
    );
}

var QReturnValue;
if (typeof ReturnValue !== "undefined") {
    QReturnValue = ReturnValue;
} else {
    QReturnValue = function (value) {
        this.value = value;
    };
}

// long stack traces

Q.longStackJumpLimit = 1;

var STACK_JUMP_SEPARATOR = "From previous event:";

function makeStackTraceLong(error, promise) {
    // If possible (that is, if in V8), transform the error stack
    // trace by removing Node and Q cruft, then concatenating with
    // the stack trace of the promise we are ``done``ing. See #57.
    if (promise.stack &&
        typeof error === "object" &&
        error !== null &&
        error.stack &&
        error.stack.indexOf(STACK_JUMP_SEPARATOR) === -1
    ) {
        error.stack = filterStackString(error.stack) +
            "\n" + STACK_JUMP_SEPARATOR + "\n" +
            filterStackString(promise.stack);
    }
}

function filterStackString(stackString) {
    var lines = stackString.split("\n");
    var desiredLines = [];
    for (var i = 0; i < lines.length; ++i) {
        var line = lines[i];

        if (!isInternalFrame(line) && !isNodeFrame(line)) {
            desiredLines.push(line);
        }
    }
    return desiredLines.join("\n");
}

function isNodeFrame(stackLine) {
    return stackLine.indexOf("(module.js:") !== -1 ||
           stackLine.indexOf("(node.js:") !== -1;
}

function isInternalFrame(stackLine) {
    var pieces = /at .+ \((.*):(\d+):\d+\)/.exec(stackLine);

    if (!pieces) {
        return false;
    }

    var fileName = pieces[1];
    var lineNumber = pieces[2];

    return fileName === qFileName &&
        lineNumber >= qStartingLine &&
        lineNumber <= qEndingLine;
}

// discover own file name and line number range for filtering stack
// traces
function captureLine() {
    if (Error.captureStackTrace) {
        var fileName, lineNumber;

        var oldPrepareStackTrace = Error.prepareStackTrace;

        Error.prepareStackTrace = function (error, frames) {
            fileName = frames[1].getFileName();
            lineNumber = frames[1].getLineNumber();
        };

        // teases call of temporary prepareStackTrace
        // JSHint and Closure Compiler generate known warnings here
        /*jshint expr: true */
        new Error().stack;

        Error.prepareStackTrace = oldPrepareStackTrace;
        qFileName = fileName;
        return lineNumber;
    }
}

function deprecate(callback, name, alternative) {
    return function () {
        if (typeof console !== "undefined" && typeof console.warn === "function") {
            console.warn(name + " is deprecated, use " + alternative + " instead.", new Error("").stack);
        }
        return callback.apply(callback, arguments);
    };
}

// end of shims
// beginning of real work

/**
 * Creates fulfilled promises from non-promises,
 * Passes Q promises through,
 * Coerces CommonJS/Promises/A+ promises to Q promises.
 */
function Q(value) {
    return resolve(value);
}

/**
 * Performs a task in a future turn of the event loop.
 * @param {Function} task
 */
Q.nextTick = nextTick;

/**
 * Constructs a {promise, resolve} object.
 *
 * The resolver is a callback to invoke with a more resolved value for the
 * promise. To fulfill the promise, invoke the resolver with any value that is
 * not a function. To reject the promise, invoke the resolver with a rejection
 * object. To put the promise in the same state as another promise, invoke the
 * resolver with that other promise.
 */
Q.defer = defer;
function defer() {
    // if "pending" is an "Array", that indicates that the promise has not yet
    // been resolved.  If it is "undefined", it has been resolved.  Each
    // element of the pending array is itself an array of complete arguments to
    // forward to the resolved promise.  We coerce the resolution value to a
    // promise using the ref promise because it handles both fully
    // resolved values and other promises gracefully.
    var pending = [], progressListeners = [], value;

    var deferred = object_create(defer.prototype);
    var promise = object_create(makePromise.prototype);

    promise.promiseDispatch = function (resolve, op, operands) {
        var args = array_slice(arguments);
        if (pending) {
            pending.push(args);
            if (op === "when" && operands[1]) { // progress operand
                progressListeners.push(operands[1]);
            }
        } else {
            nextTick(function () {
                value.promiseDispatch.apply(value, args);
            });
        }
    };

    promise.valueOf = function () {
        if (pending) {
            return promise;
        }
        value = valueOf(value); // shorten chain
        return value;
    };

    if (Error.captureStackTrace && Q.longStackJumpLimit > 0) {
        Error.captureStackTrace(promise, defer);

        // Reify the stack into a string by using the accessor; this prevents
        // memory leaks as per GH-111. At the same time, cut off the first line;
        // it's always just "[object Promise]\n", as per the `toString`.
        promise.stack = promise.stack.substring(promise.stack.indexOf("\n") + 1);
    }

    function become(resolvedValue) {
        if (!pending) {
            return;
        }
        value = resolve(resolvedValue);
        array_reduce(pending, function (undefined, pending) {
            nextTick(function () {
                value.promiseDispatch.apply(value, pending);
            });
        }, void 0);
        pending = void 0;
        progressListeners = void 0;
    }

    deferred.promise = promise;
    deferred.resolve = become;
    deferred.fulfill = function (value) {
        become(fulfill(value));
    };
    deferred.reject = function (exception) {
        become(reject(exception));
    };
    deferred.notify = function (progress) {
        if (pending) {
            array_reduce(progressListeners, function (undefined, progressListener) {
                nextTick(function () {
                    progressListener(progress);
                });
            }, void 0);
        }
    };

    return deferred;
}

/**
 * Creates a Node-style callback that will resolve or reject the deferred
 * promise.
 * @returns a nodeback
 */
defer.prototype.makeNodeResolver = function () {
    var self = this;
    return function (error, value) {
        if (error) {
            self.reject(error);
        } else if (arguments.length > 2) {
            self.resolve(array_slice(arguments, 1));
        } else {
            self.resolve(value);
        }
    };
};

/**
 * @param makePromise {Function} a function that returns nothing and accepts
 * the resolve, reject, and notify functions for a deferred.
 * @returns a promise that may be resolved with the given resolve and reject
 * functions, or rejected by a thrown exception in makePromise
 */
Q.promise = promise;
function promise(makePromise) {
    var deferred = defer();
    fcall(
        makePromise,
        deferred.resolve,
        deferred.reject,
        deferred.notify
    ).fail(deferred.reject);
    return deferred.promise;
}

/**
 * Constructs a Promise with a promise descriptor object and optional fallback
 * function.  The descriptor contains methods like when(rejected), get(name),
 * put(name, value), post(name, args), and delete(name), which all
 * return either a value, a promise for a value, or a rejection.  The fallback
 * accepts the operation name, a resolver, and any further arguments that would
 * have been forwarded to the appropriate method above had a method been
 * provided with the proper name.  The API makes no guarantees about the nature
 * of the returned object, apart from that it is usable whereever promises are
 * bought and sold.
 */
Q.makePromise = makePromise;
function makePromise(descriptor, fallback, valueOf, exception, isException) {
    if (fallback === void 0) {
        fallback = function (op) {
            return reject(new Error("Promise does not support operation: " + op));
        };
    }

    var promise = object_create(makePromise.prototype);

    promise.promiseDispatch = function (resolve, op, args) {
        var result;
        try {
            if (descriptor[op]) {
                result = descriptor[op].apply(promise, args);
            } else {
                result = fallback.call(promise, op, args);
            }
        } catch (exception) {
            result = reject(exception);
        }
        if (resolve) {
            resolve(result);
        }
    };

    if (valueOf) {
        promise.valueOf = valueOf;
    }

    if (isException) {
        promise.exception = exception;
    }

    return promise;
}

// provide thenables, CommonJS/Promises/A
makePromise.prototype.then = function (fulfilled, rejected, progressed) {
    return when(this, fulfilled, rejected, progressed);
};

makePromise.prototype.thenResolve = function (value) {
    return when(this, function () { return value; });
};

// Chainable methods
array_reduce(
    [
        "isFulfilled", "isRejected", "isPending",
        "dispatch",
        "when", "spread",
        "get", "put", "set", "del", "delete",
        "post", "send", "invoke",
        "keys",
        "fapply", "fcall", "fbind",
        "all", "allResolved",
        "timeout", "delay",
        "catch", "finally", "fail", "fin", "progress", "done",
        "nfcall", "nfapply", "nfbind", "denodeify", "nbind",
        "ncall", "napply", "nbind",
        "npost", "nsend", "ninvoke",
        "nodeify"
    ],
    function (undefined, name) {
        makePromise.prototype[name] = function () {
            return Q[name].apply(
                Q,
                [this].concat(array_slice(arguments))
            );
        };
    },
    void 0
);

makePromise.prototype.toSource = function () {
    return this.toString();
};

makePromise.prototype.toString = function () {
    return "[object Promise]";
};

/**
 * If an object is not a promise, it is as "near" as possible.
 * If a promise is rejected, it is as "near" as possible too.
 * If it’s a fulfilled promise, the fulfillment value is nearer.
 * If it’s a deferred promise and the deferred has been resolved, the
 * resolution is "nearer".
 * @param object
 * @returns most resolved (nearest) form of the object
 */
Q.nearer = valueOf;
function valueOf(value) {
    if (isPromise(value)) {
        return value.valueOf();
    }
    return value;
}

/**
 * @returns whether the given object is a promise.
 * Otherwise it is a fulfilled value.
 */
Q.isPromise = isPromise;
function isPromise(object) {
    return object && typeof object.promiseDispatch === "function";
}

Q.isPromiseAlike = isPromiseAlike;
function isPromiseAlike(object) {
    return object && typeof object.then === "function";
}

/**
 * @returns whether the given object is a pending promise, meaning not
 * fulfilled or rejected.
 */
Q.isPending = isPending;
function isPending(object) {
    return !isFulfilled(object) && !isRejected(object);
}

/**
 * @returns whether the given object is a value or fulfilled
 * promise.
 */
Q.isFulfilled = isFulfilled;
function isFulfilled(object) {
    return !isPromiseAlike(valueOf(object));
}

/**
 * @returns whether the given object is a rejected promise.
 */
Q.isRejected = isRejected;
function isRejected(object) {
    object = valueOf(object);
    return isPromise(object) && 'exception' in object;
}

var rejections = [];
var errors = [];
var errorsDisplayed;
function displayErrors() {
    if (
        !errorsDisplayed &&
        typeof window !== "undefined" &&
        !window.Touch &&
        window.console
    ) {
        // This promise library consumes exceptions thrown in handlers so
        // they can be handled by a subsequent promise.  The rejected
        // promises get added to this array when they are created, and
        // removed when they are handled.
        console.log("Should be empty:", errors);
    }
    errorsDisplayed = true;
}

// Show unhandled rejection if Node exits without handling an outstanding
// rejection.  (Note that Browserify presently produces a process global
// without the Emitter on interface)
if (typeof process !== "undefined" && process.on) {
    process.on("exit", function () {
        for (var i = 0; i < errors.length; i++) {
            var error = errors[i];
            if (error && typeof error.stack !== "undefined") {
                console.warn("Unhandled rejected promise:", error.stack);
            } else {
                console.warn("Unhandled rejected promise (no stack):", error);
            }
        }
    });
}

/**
 * Constructs a rejected promise.
 * @param exception value describing the failure
 */
Q.reject = reject;
function reject(exception) {
    var rejection = makePromise({
        "when": function (rejected) {
            // note that the error has been handled
            if (rejected) {
                var at = array_indexOf(rejections, this);
                if (at !== -1) {
                    errors.splice(at, 1);
                    rejections.splice(at, 1);
                }
            }
            return rejected ? rejected(exception) : this;
        }
    }, function fallback() {
        return reject(exception);
    }, function valueOf() {
        return this;
    }, exception, true);
    // note that the error has not been handled
    displayErrors();
    rejections.push(rejection);
    errors.push(exception);
    return rejection;
}

/**
 * Constructs a fulfilled promise for an immediate reference.
 * @param value immediate reference
 */
Q.fulfill = fulfill;
function fulfill(object) {
    return makePromise({
        "when": function () {
            return object;
        },
        "get": function (name) {
            return object[name];
        },
        "set": function (name, value) {
            object[name] = value;
        },
        "delete": function (name) {
            delete object[name];
        },
        "post": function (name, args) {
            // Mark Miller proposes that post with no name should apply a
            // promised function.
            if (name == null) { // iff name is null or undefined
                return object.apply(void 0, args);
            } else {
                return object[name].apply(object, args);
            }
        },
        "apply": function (thisP, args) {
            return object.apply(thisP, args);
        },
        "keys": function () {
            return object_keys(object);
        }
    }, void 0, function valueOf() {
        return object;
    });
}

/**
 * Constructs a promise for an immediate reference, passes promises through, or
 * coerces promises from different systems.
 * @param value immediate reference or promise
 */
Q.resolve = resolve;
function resolve(value) {
    // If the object is already a Promise, return it directly.  This enables
    // the resolve function to both be used to created references from objects,
    // but to tolerably coerce non-promises to promises.
    if (isPromise(value)) {
        return value;
    }
    // In order to break infinite recursion or loops between `then` and
    // `resolve`, it is necessary to attempt to extract fulfilled values
    // out of foreign promise implementations before attempting to wrap
    // them as unresolved promises.  It is my hope that other
    // implementations will implement `valueOf` to synchronously extract
    // the fulfillment value from their fulfilled promises.  If the
    // other promise library does not implement `valueOf`, the
    // implementations on primordial prototypes are harmless.
    value = valueOf(value);
    // assimilate thenables, CommonJS/Promises/A+
    if (isPromiseAlike(value)) {
        return coerce(value);
    } else {
        return fulfill(value);
    }
}

/**
 * Converts thenables to Q promises.
 * @param promise thenable promise
 * @returns a Q promise
 */
function coerce(promise) {
    var deferred = defer();
    nextTick(function () {
        try {
            promise.then(deferred.resolve, deferred.reject, deferred.notify);
        } catch (exception) {
            deferred.reject(exception);
        }
    });
    return deferred.promise;
}

/**
 * Annotates an object such that it will never be
 * transferred away from this process over any promise
 * communication channel.
 * @param object
 * @returns promise a wrapping of that object that
 * additionally responds to the "isDef" message
 * without a rejection.
 */
Q.master = master;
function master(object) {
    return makePromise({
        "isDef": function () {}
    }, function fallback(op, args) {
        return dispatch(object, op, args);
    }, function () {
        return valueOf(object);
    });
}

/**
 * Registers an observer on a promise.
 *
 * Guarantees:
 *
 * 1. that fulfilled and rejected will be called only once.
 * 2. that either the fulfilled callback or the rejected callback will be
 *    called, but not both.
 * 3. that fulfilled and rejected will not be called in this turn.
 *
 * @param value      promise or immediate reference to observe
 * @param fulfilled  function to be called with the fulfilled value
 * @param rejected   function to be called with the rejection exception
 * @param progressed function to be called on any progress notifications
 * @return promise for the return value from the invoked callback
 */
Q.when = when;
function when(value, fulfilled, rejected, progressed) {
    var deferred = defer();
    var done = false;   // ensure the untrusted promise makes at most a
                        // single call to one of the callbacks

    function _fulfilled(value) {
        try {
            return typeof fulfilled === "function" ? fulfilled(value) : value;
        } catch (exception) {
            return reject(exception);
        }
    }

    function _rejected(exception) {
        if (typeof rejected === "function") {
            makeStackTraceLong(exception, resolvedValue);
            try {
                return rejected(exception);
            } catch (newException) {
                return reject(newException);
            }
        }
        return reject(exception);
    }

    function _progressed(value) {
        return typeof progressed === "function" ? progressed(value) : value;
    }

    var resolvedValue = resolve(value);
    nextTick(function () {
        resolvedValue.promiseDispatch(function (value) {
            if (done) {
                return;
            }
            done = true;

            deferred.resolve(_fulfilled(value));
        }, "when", [function (exception) {
            if (done) {
                return;
            }
            done = true;

            deferred.resolve(_rejected(exception));
        }]);
    });

    // Progress propagator need to be attached in the current tick.
    resolvedValue.promiseDispatch(void 0, "when", [void 0, function (value) {
        var newValue;
        var threw = false;
        try {
            newValue = _progressed(value);
        } catch (e) {
            threw = true;
            if (Q.onerror) {
                Q.onerror(e);
            } else {
                throw e;
            }
        }

        if (!threw) {
            deferred.notify(newValue);
        }
    }]);

    return deferred.promise;
}

/**
 * Spreads the values of a promised array of arguments into the
 * fulfillment callback.
 * @param fulfilled callback that receives variadic arguments from the
 * promised array
 * @param rejected callback that receives the exception if the promise
 * is rejected.
 * @returns a promise for the return value or thrown exception of
 * either callback.
 */
Q.spread = spread;
function spread(promise, fulfilled, rejected) {
    return when(promise, function (valuesOrPromises) {
        return all(valuesOrPromises).then(function (values) {
            return fulfilled.apply(void 0, values);
        }, rejected);
    }, rejected);
}

/**
 * The async function is a decorator for generator functions, turning
 * them into asynchronous generators.  This presently only works in
 * Firefox/Spidermonkey, however, this code does not cause syntax
 * errors in older engines.  This code should continue to work and
 * will in fact improve over time as the language improves.
 *
 * Decorates a generator function such that:
 *  - it may yield promises
 *  - execution will continue when that promise is fulfilled
 *  - the value of the yield expression will be the fulfilled value
 *  - it returns a promise for the return value (when the generator
 *    stops iterating)
 *  - the decorated function returns a promise for the return value
 *    of the generator or the first rejected promise among those
 *    yielded.
 *  - if an error is thrown in the generator, it propagates through
 *    every following yield until it is caught, or until it escapes
 *    the generator function altogether, and is translated into a
 *    rejection for the promise returned by the decorated generator.
 *  - in present implementations of generators, when a generator
 *    function is complete, it throws ``StopIteration``, ``return`` is
 *    a syntax error in the presence of ``yield``, so there is no
 *    observable return value. There is a proposal[1] to add support
 *    for ``return``, which would permit the value to be carried by a
 *    ``StopIteration`` instance, in which case it would fulfill the
 *    promise returned by the asynchronous generator.  This can be
 *    emulated today by throwing StopIteration explicitly with a value
 *    property.
 *
 *  [1]: http://wiki.ecmascript.org/doku.php?id=strawman:async_functions#reference_implementation
 *
 */
Q.async = async;
function async(makeGenerator) {
    return function () {
        // when verb is "send", arg is a value
        // when verb is "throw", arg is an exception
        function continuer(verb, arg) {
            var result;
            try {
                result = generator[verb](arg);
            } catch (exception) {
                if (isStopIteration(exception)) {
                    return exception.value;
                } else {
                    return reject(exception);
                }
            }
            return when(result, callback, errback);
        }
        var generator = makeGenerator.apply(this, arguments);
        var callback = continuer.bind(continuer, "send");
        var errback = continuer.bind(continuer, "throw");
        return callback();
    };
}

/**
 * Throws a ReturnValue exception to stop an asynchronous generator.
 * Only useful presently in Firefox/SpiderMonkey since generators are
 * implemented.
 * @param value the return value for the surrounding generator
 * @throws ReturnValue exception with the value.
 * @example
 * Q.async(function () {
 *      var foo = yield getFooPromise();
 *      var bar = yield getBarPromise();
 *      Q.return(foo + bar);
 * })
 */
Q['return'] = _return;
function _return(value) {
    throw new QReturnValue(value);
}

/**
 * The promised function decorator ensures that any promise arguments
 * are resolved and passed as values (`this` is also resolved and passed
 * as a value).  It will also ensure that the result of a function is
 * always a promise.
 *
 * @example
 * var add = Q.promised(function (a, b) {
 *     return a + b;
 * });
 * add(Q.resolve(a), Q.resolve(B));
 *
 * @param {function} callback The function to decorate
 * @returns {function} a function that has been decorated.
 */
Q.promised = promised;
function promised(callback) {
    return function () {
        return spread([this, all(arguments)], function (self, args) {
            return callback.apply(self, args);
        });
    };
}

/**
 * sends a message to a value in a future turn
 * @param object* the recipient
 * @param op the name of the message operation, e.g., "when",
 * @param args further arguments to be forwarded to the operation
 * @returns result {Promise} a promise for the result of the operation
 */
Q.dispatch = dispatch;
function dispatch(object, op, args) {
    var deferred = defer();
    nextTick(function () {
        resolve(object).promiseDispatch(deferred.resolve, op, args);
    });
    return deferred.promise;
}

/**
 * Constructs a promise method that can be used to safely observe resolution of
 * a promise for an arbitrarily named method like "propfind" in a future turn.
 *
 * "dispatcher" constructs methods like "get(promise, name)" and "put(promise)".
 */
Q.dispatcher = dispatcher;
function dispatcher(op) {
    return function (object) {
        var args = array_slice(arguments, 1);
        return dispatch(object, op, args);
    };
}

/**
 * Gets the value of a property in a future turn.
 * @param object    promise or immediate reference for target object
 * @param name      name of property to get
 * @return promise for the property value
 */
Q.get = dispatcher("get");

/**
 * Sets the value of a property in a future turn.
 * @param object    promise or immediate reference for object object
 * @param name      name of property to set
 * @param value     new value of property
 * @return promise for the return value
 */
Q.set = dispatcher("set");

/**
 * Deletes a property in a future turn.
 * @param object    promise or immediate reference for target object
 * @param name      name of property to delete
 * @return promise for the return value
 */
Q["delete"] = // XXX experimental
Q.del = dispatcher("delete");

/**
 * Invokes a method in a future turn.
 * @param object    promise or immediate reference for target object
 * @param name      name of method to invoke
 * @param value     a value to post, typically an array of
 *                  invocation arguments for promises that
 *                  are ultimately backed with `resolve` values,
 *                  as opposed to those backed with URLs
 *                  wherein the posted value can be any
 *                  JSON serializable object.
 * @return promise for the return value
 */
// bound locally because it is used by other methods
var post = Q.post = dispatcher("post");

/**
 * Invokes a method in a future turn.
 * @param object    promise or immediate reference for target object
 * @param name      name of method to invoke
 * @param ...args   array of invocation arguments
 * @return promise for the return value
 */
Q.send = send;
Q.invoke = send; // synonyms
function send(value, name) {
    var args = array_slice(arguments, 2);
    return post(value, name, args);
}

/**
 * Applies the promised function in a future turn.
 * @param object    promise or immediate reference for target function
 * @param args      array of application arguments
 */
Q.fapply = fapply;
function fapply(value, args) {
    return dispatch(value, "apply", [void 0, args]);
}

/**
 * Calls the promised function in a future turn.
 * @param object    promise or immediate reference for target function
 * @param ...args   array of application arguments
 */
Q["try"] = fcall; // XXX experimental
Q.fcall = fcall;
function fcall(value) {
    var args = array_slice(arguments, 1);
    return fapply(value, args);
}

/**
 * Binds the promised function, transforming return values into a fulfilled
 * promise and thrown errors into a rejected one.
 * @param object    promise or immediate reference for target function
 * @param ...args   array of application arguments
 */
Q.fbind = fbind;
function fbind(value) {
    var args = array_slice(arguments, 1);
    return function fbound() {
        var allArgs = args.concat(array_slice(arguments));
        return dispatch(value, "apply", [this, allArgs]);
    };
}

/**
 * Requests the names of the owned properties of a promised
 * object in a future turn.
 * @param object    promise or immediate reference for target object
 * @return promise for the keys of the eventually resolved object
 */
Q.keys = dispatcher("keys");

/**
 * Turns an array of promises into a promise for an array.  If any of
 * the promises gets rejected, the whole array is rejected immediately.
 * @param {Array*} an array (or promise for an array) of values (or
 * promises for values)
 * @returns a promise for an array of the corresponding values
 */
// By Mark Miller
// http://wiki.ecmascript.org/doku.php?id=strawman:concurrency&rev=1308776521#allfulfilled
Q.all = all;
function all(promises) {
    return when(promises, function (promises) {
        var countDown = promises.length;
        if (countDown === 0) {
            return resolve(promises);
        }
        var deferred = defer();
        array_reduce(promises, function (undefined, promise, index) {
            if (isFulfilled(promise)) {
                promises[index] = valueOf(promise);
                if (--countDown === 0) {
                    deferred.resolve(promises);
                }
            } else {
                when(promise, function (value) {
                    promises[index] = value;
                    if (--countDown === 0) {
                        deferred.resolve(promises);
                    }
                })
                .fail(deferred.reject);
            }
        }, void 0);
        return deferred.promise;
    });
}

/**
 * Waits for all promises to be resolved, either fulfilled or
 * rejected.  This is distinct from `all` since that would stop
 * waiting at the first rejection.  The promise returned by
 * `allResolved` will never be rejected.
 * @param promises a promise for an array (or an array) of promises
 * (or values)
 * @return a promise for an array of promises
 */
Q.allResolved = allResolved;
function allResolved(promises) {
    return when(promises, function (promises) {
        promises = array_map(promises, resolve);
        return when(all(array_map(promises, function (promise) {
            return when(promise, noop, noop);
        })), function () {
            return promises;
        });
    });
}

/**
 * Captures the failure of a promise, giving an oportunity to recover
 * with a callback.  If the given promise is fulfilled, the returned
 * promise is fulfilled.
 * @param {Any*} promise for something
 * @param {Function} callback to fulfill the returned promise if the
 * given promise is rejected
 * @returns a promise for the return value of the callback
 */
Q["catch"] = // XXX experimental
Q.fail = fail;
function fail(promise, rejected) {
    return when(promise, void 0, rejected);
}

/**
 * Attaches a listener that can respond to progress notifications from a
 * promise's originating deferred. This listener receives the exact arguments
 * passed to ``deferred.notify``.
 * @param {Any*} promise for something
 * @param {Function} callback to receive any progress notifications
 * @returns the given promise, unchanged
 */
Q.progress = progress;
function progress(promise, progressed) {
    return when(promise, void 0, void 0, progressed);
}

/**
 * Provides an opportunity to observe the rejection of a promise,
 * regardless of whether the promise is fulfilled or rejected.  Forwards
 * the resolution to the returned promise when the callback is done.
 * The callback can return a promise to defer completion.
 * @param {Any*} promise
 * @param {Function} callback to observe the resolution of the given
 * promise, takes no arguments.
 * @returns a promise for the resolution of the given promise when
 * ``fin`` is done.
 */
Q["finally"] = // XXX experimental
Q.fin = fin;
function fin(promise, callback) {
    return when(promise, function (value) {
        return when(callback(), function () {
            return value;
        });
    }, function (exception) {
        return when(callback(), function () {
            return reject(exception);
        });
    });
}

/**
 * Terminates a chain of promises, forcing rejections to be
 * thrown as exceptions.
 * @param {Any*} promise at the end of a chain of promises
 * @returns nothing
 */
Q.done = done;
function done(promise, fulfilled, rejected, progress) {
    var onUnhandledError = function (error) {
        // forward to a future turn so that ``when``
        // does not catch it and turn it into a rejection.
        nextTick(function () {
            makeStackTraceLong(error, promise);

            if (Q.onerror) {
                Q.onerror(error);
            } else {
                throw error;
            }
        });
    };

    // Avoid unnecessary `nextTick`ing via an unnecessary `when`.
    var promiseToHandle = fulfilled || rejected || progress ?
        when(promise, fulfilled, rejected, progress) :
        promise;

    if (typeof process === "object" && process && process.domain) {
        onUnhandledError = process.domain.bind(onUnhandledError);
    }
    fail(promiseToHandle, onUnhandledError);
}

/**
 * Causes a promise to be rejected if it does not get fulfilled before
 * some milliseconds time out.
 * @param {Any*} promise
 * @param {Number} milliseconds timeout
 * @returns a promise for the resolution of the given promise if it is
 * fulfilled before the timeout, otherwise rejected.
 */
Q.timeout = timeout;
function timeout(promise, ms) {
    var deferred = defer();
    var timeoutId = setTimeout(function () {
        deferred.reject(new Error("Timed out after " + ms + " ms"));
    }, ms);

    when(promise, function (value) {
        clearTimeout(timeoutId);
        deferred.resolve(value);
    }, function (exception) {
        clearTimeout(timeoutId);
        deferred.reject(exception);
    });

    return deferred.promise;
}

/**
 * Returns a promise for the given value (or promised value) after some
 * milliseconds.
 * @param {Any*} promise
 * @param {Number} milliseconds
 * @returns a promise for the resolution of the given promise after some
 * time has elapsed.
 */
Q.delay = delay;
function delay(promise, timeout) {
    if (timeout === void 0) {
        timeout = promise;
        promise = void 0;
    }
    var deferred = defer();
    setTimeout(function () {
        deferred.resolve(promise);
    }, timeout);
    return deferred.promise;
}

/**
 * Passes a continuation to a Node function, which is called with the given
 * arguments provided as an array, and returns a promise.
 *
 *      Q.nfapply(FS.readFile, [__filename])
 *      .then(function (content) {
 *      })
 *
 */
Q.nfapply = nfapply;
function nfapply(callback, args) {
    var nodeArgs = array_slice(args);
    var deferred = defer();
    nodeArgs.push(deferred.makeNodeResolver());

    fapply(callback, nodeArgs).fail(deferred.reject);
    return deferred.promise;
}

/**
 * Passes a continuation to a Node function, which is called with the given
 * arguments provided individually, and returns a promise.
 *
 *      Q.nfcall(FS.readFile, __filename)
 *      .then(function (content) {
 *      })
 *
 */
Q.nfcall = nfcall;
function nfcall(callback/*, ...args */) {
    var nodeArgs = array_slice(arguments, 1);
    var deferred = defer();
    nodeArgs.push(deferred.makeNodeResolver());

    fapply(callback, nodeArgs).fail(deferred.reject);
    return deferred.promise;
}

/**
 * Wraps a NodeJS continuation passing function and returns an equivalent
 * version that returns a promise.
 *
 *      Q.nfbind(FS.readFile, __filename)("utf-8")
 *      .then(console.log)
 *      .done()
 *
 */
Q.nfbind = nfbind;
Q.denodeify = Q.nfbind; // synonyms
function nfbind(callback/*, ...args */) {
    var baseArgs = array_slice(arguments, 1);
    return function () {
        var nodeArgs = baseArgs.concat(array_slice(arguments));
        var deferred = defer();
        nodeArgs.push(deferred.makeNodeResolver());

        fapply(callback, nodeArgs).fail(deferred.reject);
        return deferred.promise;
    };
}

Q.nbind = nbind;
function nbind(callback/*, ... args*/) {
    var baseArgs = array_slice(arguments, 1);
    return function () {
        var nodeArgs = baseArgs.concat(array_slice(arguments));
        var deferred = defer();
        nodeArgs.push(deferred.makeNodeResolver());

        var thisArg = this;
        function bound() {
            return callback.apply(thisArg, arguments);
        }

        fapply(bound, nodeArgs).fail(deferred.reject);
        return deferred.promise;
    };
}

/**
 * Calls a method of a Node-style object that accepts a Node-style
 * callback with a given array of arguments, plus a provided callback.
 * @param object an object that has the named method
 * @param {String} name name of the method of object
 * @param {Array} args arguments to pass to the method; the callback
 * will be provided by Q and appended to these arguments.
 * @returns a promise for the value or error
 */
Q.npost = npost;
function npost(object, name, args) {
    var nodeArgs = array_slice(args || []);
    var deferred = defer();
    nodeArgs.push(deferred.makeNodeResolver());

    post(object, name, nodeArgs).fail(deferred.reject);
    return deferred.promise;
}

/**
 * Calls a method of a Node-style object that accepts a Node-style
 * callback, forwarding the given variadic arguments, plus a provided
 * callback argument.
 * @param object an object that has the named method
 * @param {String} name name of the method of object
 * @param ...args arguments to pass to the method; the callback will
 * be provided by Q and appended to these arguments.
 * @returns a promise for the value or error
 */
Q.nsend = nsend;
Q.ninvoke = Q.nsend; // synonyms
function nsend(object, name /*, ...args*/) {
    var nodeArgs = array_slice(arguments, 2);
    var deferred = defer();
    nodeArgs.push(deferred.makeNodeResolver());
    post(object, name, nodeArgs).fail(deferred.reject);
    return deferred.promise;
}

Q.nodeify = nodeify;
function nodeify(promise, nodeback) {
    if (nodeback) {
        promise.then(function (value) {
            nextTick(function () {
                nodeback(null, value);
            });
        }, function (error) {
            nextTick(function () {
                nodeback(error);
            });
        });
    } else {
        return promise;
    }
}

// All code before this point will be filtered from stack traces.
var qEndingLine = captureLine();

return Q;

});

/*
 Copyright 2011 Abdulla Abdurakhmanov
 Original sources are available at https://code.google.com/p/x2js/

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

function X2JS(matchers, attrPrefix, ignoreRoot) {
    if (attrPrefix === null || attrPrefix === undefined) {
        attrPrefix = "_";
    }
    
    if (ignoreRoot === null || ignoreRoot === undefined) {
        ignoreRoot = false;
    }
    
	var VERSION = "1.0.11";
	var escapeMode = false;

	var DOMNodeTypes = {
		ELEMENT_NODE 	   : 1,
		TEXT_NODE    	   : 3,
		CDATA_SECTION_NODE : 4,
		COMMENT_NODE       : 8,
		DOCUMENT_NODE 	   : 9
	};
	
	function getNodeLocalName( node ) {
		var nodeLocalName = node.localName;			
		if(nodeLocalName == null) // Yeah, this is IE!! 
			nodeLocalName = node.baseName;
		if(nodeLocalName == null || nodeLocalName=="") // =="" is IE too
			nodeLocalName = node.nodeName;
		return nodeLocalName;
	}
	
	function getNodePrefix(node) {
		return node.prefix;
	}
		
	function escapeXmlChars(str) {
		if(typeof(str) == "string")
			return str.replace(/&/g, '&amp;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/"/g, '&quot;').replace(/'/g, '&#x27;').replace(/\//g, '&#x2F;');
		else
			return str;
	}

	function unescapeXmlChars(str) {
		return str.replace(/&amp;/g, '&').replace(/&lt;/g, '<').replace(/&gt;/g, '>').replace(/&quot;/g, '"').replace(/&#x27;/g, "'").replace(/&#x2F;/g, '\/')
	}	

	function parseDOMChildren( node ) {
		if(node.nodeType == DOMNodeTypes.DOCUMENT_NODE) {
			var result,
			    child = node.firstChild,
			    i,
			    len; 
			
			// get the first node that isn't a comment
			for(i = 0, len = node.childNodes.length; i < len; i += 1) {
			   if (node.childNodes[i].nodeType !== DOMNodeTypes.COMMENT_NODE) {
			       child = node.childNodes[i];
			       break;
			   } 
			}
			
			if ( ignoreRoot ) {
			    result = parseDOMChildren(child);
			} else {
			    result = {};
			    var childName = getNodeLocalName(child);
                result[childName] = parseDOMChildren(child);
			}
			
			return result;
		}
		else
		if(node.nodeType == DOMNodeTypes.ELEMENT_NODE) {
			var result = new Object;
			result.__cnt=0;
			
			var nodeChildren = node.childNodes;
			
			// Children nodes
			for(var cidx=0; cidx <nodeChildren.length; cidx++) {
				var child = nodeChildren.item(cidx); // nodeChildren[cidx];
				var childName = getNodeLocalName(child);
				
				result.__cnt++;
				if(result[childName] == null) {
					result[childName] = parseDOMChildren(child);
					result[childName+"_asArray"] = new Array(1);
					result[childName+"_asArray"][0] = result[childName];
				}
				else {
					if(result[childName] != null) {
						if( !(result[childName] instanceof Array)) {
							var tmpObj = result[childName];
							result[childName] = new Array();
							result[childName][0] = tmpObj;
							
							result[childName+"_asArray"] = result[childName];
						}
					}
					var aridx = 0;
					while(result[childName][aridx]!=null) aridx++;
					(result[childName])[aridx] = parseDOMChildren(child);
				}			
			}
			
			// Attributes
			for(var aidx=0; aidx <node.attributes.length; aidx++) {
				var attr = node.attributes.item(aidx); // [aidx];
				result.__cnt++;
				
				var value2 = attr.value;
				for(var m=0, ml=matchers.length; m < ml; m++) {
				    var matchobj = matchers[m];
				    if (matchobj.test.call(this, attr.value))
				        value2 = matchobj.converter.call(this, attr.value);
				}
				
				result[attrPrefix+attr.name]=value2;
			}
			
			// Node namespace prefix
			var nodePrefix = getNodePrefix(node);
			if(nodePrefix!=null && nodePrefix!="") {
				result.__cnt++;
				result.__prefix=nodePrefix;
			}
			
			if( result.__cnt == 1 && result["#text"]!=null  ) {
				result = result["#text"];
			} 
			
			if(result["#text"]!=null) {
				result.__text = result["#text"];
				if(escapeMode)
					result.__text = unescapeXmlChars(result.__text)
				delete result["#text"];
				delete result["#text_asArray"];
			}
			if(result["#cdata-section"]!=null) {
				result.__cdata = result["#cdata-section"];
				delete result["#cdata-section"];
				delete result["#cdata-section_asArray"];
			}
			
			if(result.__text!=null || result.__cdata!=null) {
				result.toString = function() {
					return (this.__text!=null? this.__text:'')+( this.__cdata!=null ? this.__cdata:'');
				}
			}
			return result;
		}
		else
		if(node.nodeType == DOMNodeTypes.TEXT_NODE || node.nodeType == DOMNodeTypes.CDATA_SECTION_NODE) {
			return node.nodeValue;
		}	
		else
		if(node.nodeType == DOMNodeTypes.COMMENT_NODE) {
		    return null;
		}
	}
	
	function startTag(jsonObj, element, attrList, closed) {
		var resultStr = "<"+ ( (jsonObj!=null && jsonObj.__prefix!=null)? (jsonObj.__prefix+":"):"") + element;
		if(attrList!=null) {
			for(var aidx = 0; aidx < attrList.length; aidx++) {
				var attrName = attrList[aidx];
				var attrVal = jsonObj[attrName];
				resultStr+=" "+attrName.substr(1)+"='"+attrVal+"'";
			}
		}
		if(!closed)
			resultStr+=">";
		else
			resultStr+="/>";
		return resultStr;
	}
	
	function endTag(jsonObj,elementName) {
		return "</"+ (jsonObj.__prefix!=null? (jsonObj.__prefix+":"):"")+elementName+">";
	}
	
	function endsWith(str, suffix) {
	    return str.indexOf(suffix, str.length - suffix.length) !== -1;
	}
	
	function jsonXmlSpecialElem ( jsonObj, jsonObjField ) {
		if(endsWith(jsonObjField.toString(),("_asArray")) 
				|| jsonObjField.toString().indexOf("_")==0 
				|| (jsonObj[jsonObjField] instanceof Function) )
			return true;
		else
			return false;
	}
	
	function jsonXmlElemCount ( jsonObj ) {
		var elementsCnt = 0;
		if(jsonObj instanceof Object ) {
			for( var it in jsonObj  ) {
				if(jsonXmlSpecialElem ( jsonObj, it) )
					continue;			
				elementsCnt++;
			}
		}
		return elementsCnt;
	}
	
	function parseJSONAttributes ( jsonObj ) {
		var attrList = [];
		if(jsonObj instanceof Object ) {
			for( var ait in jsonObj  ) {
				if(ait.toString().indexOf("__")== -1 && ait.toString().indexOf("_")==0) {
					attrList.push(ait);
				}
			}
		}
		return attrList;
	}
	
	function parseJSONTextAttrs ( jsonTxtObj ) {
		var result ="";
		
		if(jsonTxtObj.__cdata!=null) {										
			result+="<![CDATA["+jsonTxtObj.__cdata+"]]>";					
		}
		
		if(jsonTxtObj.__text!=null) {			
			if(escapeMode)
				result+=escapeXmlChars(jsonTxtObj.__text);
			else
				result+=jsonTxtObj.__text;
		}
		return result
	}
	
	function parseJSONTextObject ( jsonTxtObj ) {
		var result ="";

		if( jsonTxtObj instanceof Object ) {
			result+=parseJSONTextAttrs ( jsonTxtObj )
		}
		else
			if(jsonTxtObj!=null) {
				if(escapeMode)
					result+=escapeXmlChars(jsonTxtObj);
				else
					result+=jsonTxtObj;
			}
		
		return result;
	}
	
	function parseJSONArray ( jsonArrRoot, jsonArrObj, attrList ) {
		var result = ""; 
		if(jsonArrRoot.length == 0) {
			result+=startTag(jsonArrRoot, jsonArrObj, attrList, true);
		}
		else {
			for(var arIdx = 0; arIdx < jsonArrRoot.length; arIdx++) {
				result+=startTag(jsonArrRoot[arIdx], jsonArrObj, parseJSONAttributes(jsonArrRoot[arIdx]), false);
				result+=parseJSONObject(jsonArrRoot[arIdx]);
				result+=endTag(jsonArrRoot[arIdx],jsonArrObj);						
			}
		}
		return result;
	}
	
	function parseJSONObject ( jsonObj ) {
		var result = "";	

		var elementsCnt = jsonXmlElemCount ( jsonObj );
		
		if(elementsCnt > 0) {
			for( var it in jsonObj ) {
				
				if(jsonXmlSpecialElem ( jsonObj, it) )
					continue;			
				
				var subObj = jsonObj[it];						
				
				var attrList = parseJSONAttributes( subObj )
				
				if(subObj == null || subObj == undefined) {
					result+=startTag(subObj, it, attrList, true)
				}
				else
				if(subObj instanceof Object) {
					
					if(subObj instanceof Array) {					
						result+=parseJSONArray( subObj, it, attrList )					
					}
					else {
						var subObjElementsCnt = jsonXmlElemCount ( subObj );
						if(subObjElementsCnt > 0 || subObj.__text!=null || subObj.__cdata!=null) {
							result+=startTag(subObj, it, attrList, false);
							result+=parseJSONObject(subObj);
							result+=endTag(subObj,it);
						}
						else {
							result+=startTag(subObj, it, attrList, true);
						}
					}
				}
				else {
					result+=startTag(subObj, it, attrList, false);
					result+=parseJSONTextObject(subObj);
					result+=endTag(subObj,it);
				}
			}
		}
		result+=parseJSONTextObject(jsonObj);
		
		return result;
	}
	
	this.parseXmlString = function(xmlDocStr) {
		var xmlDoc;
		if (window.DOMParser) {
			var parser=new window.DOMParser();			
			xmlDoc = parser.parseFromString( xmlDocStr, "text/xml" );
		}
		else {
			// IE :(
			if(xmlDocStr.indexOf("<?")==0) {
				xmlDocStr = xmlDocStr.substr( xmlDocStr.indexOf("?>") + 2 );
			}
			xmlDoc=new ActiveXObject("Microsoft.XMLDOM");
			xmlDoc.async="false";
			xmlDoc.loadXML(xmlDocStr);
		}
		return xmlDoc;
	}

	this.xml2json = function (xmlDoc) {
		return parseDOMChildren ( xmlDoc );
	}
	
	this.xml_str2json = function (xmlDocStr) {
		var xmlDoc = this.parseXmlString(xmlDocStr);	
		return this.xml2json(xmlDoc);
	}

	this.json2xml_str = function (jsonObj) {
		return parseJSONObject ( jsonObj );
	}

	this.json2xml = function (jsonObj) {
		var xmlDocStr = this.json2xml_str (jsonObj);
		return this.parseXmlString(xmlDocStr);
	}
	
	this.getVersion = function () {
		return VERSION;
	}		
	
	this.escapeMode = function(enabled) {
		escapeMode = enabled;
	}
}
/*
 Copyright 2011 Nathan Weber
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

/*
 * var parent,
 *     child,
 *     properties = [
                    {
                        name: 'profiles',
                        merge: false
                    }
                ];
 * 
 * parent = {};
 * parent.name = "ParentNode";
 * parent.isRoor = false;
 * parent.isArray = false;
 * parent.parent = null;
 * parent.children = [];
 * parent.properties = properties;
 * 
 * child = {};
 * child.name = "ChildNode";
 * child.isRoor = false;
 * child.isArray = true;
 * child.parent = parent;
 * child.children = null;
 * child.properties = properties;
 * parent.children.push(child);
 * 
 */

function ObjectIron(map) {
    
    var lookup;
    
    // create a list of top level items to search for
    lookup = [];
    for (i = 0, len = map.length; i < len; i += 1) {
        if (map[i].isRoot) {
            lookup.push("root");
        } else {
            lookup.push(map[i].name);
        }
    }
    
    var mergeValues = function (parentItem, childItem) {
            var name,
                parentValue,
                childValue;
            
            if (parentItem === null || childItem === null) {
                return;
            }
            
            for (name in parentItem) {
                if (parentItem.hasOwnProperty(name)) {
                    if (!childItem.hasOwnProperty(name)) {
                        childItem[name] = parentItem[name];
                    }
                }
            }
        },
    
        mapProperties = function (properties, parent, child) {
            var i,
                len,
                property,
                parentValue,
                childValue;
            
            if (properties === null || properties.length === 0) {
                return;
            }
            
            for (i = 0, len = properties.length; i < len; i += 1) {
                property = properties[i];
                
                if (parent.hasOwnProperty(property.name)) {
                    if (child.hasOwnProperty(property.name)) {
                        // check to see if we should merge
                        if (property.merge) {
                           parentValue = parent[property.name];
                           childValue = child[property.name];
                            
                            // complex objects; merge properties
                            if (typeof parentValue === 'object' && typeof childValue === 'object') {
                                mergeValues(parentValue, childValue);
                            }
                            // simple objects; merge them together
                            else {
                                child[property.name] = parentValue + childValue;
                            }
                        }
                    } else {
                        // just add the property
                        child[property.name] = parent[property.name];
                    }
                }
            }
        },
    
        mapItem = function (obj, node) {
            var item = obj,
                i,
                len,
                v,
                len2,
                array,
                childItem,
                childNode,
                property;
            
            if (item.children === null || item.children.length === 0) {
                return;
            }
            
            for (i = 0, len = item.children.length; i < len; i += 1) {
                childItem = item.children[i];
                
                if (node.hasOwnProperty(childItem.name)) {
                    if (childItem.isArray) {
                        array = node[childItem.name + "_asArray"];
                        for (v = 0, len2 = array.length; v < len2; v += 1) {
                            childNode = array[v];
                            mapProperties(item.properties, node, childNode);
                            mapItem(childItem, childNode);
                        }
                    } else {
                        childNode = node[childItem.name];
                        mapProperties(item.properties, node, childNode);
                        mapItem(childItem, childNode);
                    }
                }
            }
        },
    
        performMapping = function (source) {
            var i,
                len,
                pi,
                pp,
                item,
                node,
                array;
            
            if (source === null) {
                return;
            }
            
            if (typeof source !== 'object') {
                return;
            }
            
            // first look to see if anything cares about the root node
            for (i = 0, len = lookup.length; i < len; i += 1) {
                if (lookup[i] === "root") {
                    item = map[i];
                    node = source;
                    mapItem(item, node);
                }
            }
            
            // iterate over the objects and look for any of the items we care about
            for (pp in source) {
                if (source.hasOwnProperty(pp)) {
                    pi = lookup.indexOf(pp);
                    if (pi !== -1) {
                        item = map[pi];
                        
                        if (item.isArray) {
                            array = source[pp + "_asArray"];
                            for (i = 0, len = array.length; i < len; i += 1) {
                                node = array[i];
                                mapItem(item, node);
                            }
                        } else {
                            node = source[pp];
                            mapItem(item, node);
                        }
                    }
                    // now check this to see if he has any of the properties we care about
                    performMapping(source[pp]);
                }
            }
            
            return source;
        };
    
    return {
        run: performMapping
    };
}
